# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import time
import logging
import shutil
from unitorch.cli import CoreConfigureParser
from unitorch.cli import register_service, GenericService


@register_service("core/service/mirror_files")
class MirrorFilesService(GenericService):
    def __init__(self, config: CoreConfigureParser):
        self.config = config
        config.set_default_section("core/service/mirror_files")
        self.name = config.getoption("processname", "core_mirror_files_service")
        self.mirror_files = config.getoption("mirror_files", {})
        self.mirror_files = {k: v for k, v in self.mirror_files.items() if k != v}
        self.mirror_interval = config.getoption("mirror_interval", 120)
        self.mirror_checks = config.getoption("mirror_checks", 2)
        self.last_mtime = {src: None for src in self.mirror_files}

    def is_stable(self, path):
        """Check whether the file write has completed"""
        try:
            prev_size = os.path.getsize(path)
        except FileNotFoundError:
            return False

        for _ in range(self.mirror_checks):
            time.sleep(30)
            try:
                size = os.path.getsize(path)
            except FileNotFoundError:
                return False
            if size != prev_size:
                return False
            prev_size = size
        return True

    def mirror(self):
        for src, dst in self.mirror_files.items():
            if not os.path.exists(src):
                logging.warning(f"Source file does not exist: {src}")
                continue

            try:
                mtime = os.path.getmtime(src)
            except OSError as e:
                logging.error(f"Failed to get file modification time: {src} - {e}")
                continue

            if self.last_mtime[src] is None or mtime > self.last_mtime[src]:
                logging.info(f"File update detected: {src}")
                if self.is_stable(src):
                    os.makedirs(os.path.dirname(dst), exist_ok=True)
                    try:
                        shutil.copy2(src, dst)
                        self.last_mtime[src] = mtime
                        logging.info(f"Backup completed: {src} -> {dst}")
                    except Exception as e:
                        logging.error(f"Backup failed: {src} -> {dst} - {e}")
                else:
                    logging.info(f"File is still being written, skipping: {src}")

    def start(self, **kwargs):
        logging.info("Mirror Files Service started")
        while True:
            self.mirror()
            time.sleep(self.mirror_interval)

    def stop(self, **kwargs):
        pass

    def restart(self, **kwargs):
        pass


@register_service("core/service/mirror_folders")
class MirrorFoldersService(GenericService):
    def __init__(self, config: CoreConfigureParser):
        self.config = config
        config.set_default_section("core/service/mirror_folders")
        self.name = config.getoption("processname", "core_mirror_folders_service")
        self.mirror_folders = config.getoption(
            "mirror_folders", {}
        )  # {src_dir: dst_dir}
        self.mirror_folders = {k: v for k, v in self.mirror_folders.items() if k != v}
        self.mirror_interval = config.getoption("mirror_interval", 120)
        self.mirror_checks = config.getoption("mirror_checks", 2)

        # Store the last modification time for each file
        self.last_mtime = {}

    def is_stable(self, path):
        """Check whether the file write has completed"""
        try:
            prev_size = os.path.getsize(path)
        except FileNotFoundError:
            return False

        for _ in range(self.mirror_checks):
            time.sleep(30)
            try:
                size = os.path.getsize(path)
            except FileNotFoundError:
                return False
            if size != prev_size:
                return False
            prev_size = size
        return True

    def mirror(self):
        """Traverse all folders and synchronize"""
        for src_dir, dst_dir in self.mirror_folders.items():
            if not os.path.exists(src_dir):
                logging.warning(f"Source folder does not exist: {src_dir}")
                continue

            for root, _, files in os.walk(src_dir):
                rel_path = os.path.relpath(root, src_dir)
                target_root = os.path.join(dst_dir, rel_path)

                for file_name in files:
                    src_path = os.path.join(root, file_name)
                    dst_path = os.path.join(target_root, file_name)

                    try:
                        mtime = os.path.getmtime(src_path)
                    except OSError as e:
                        logging.error(
                            f"Failed to get file modification time: {src_path} - {e}"
                        )
                        continue

                    if (
                        self.last_mtime.get(src_path) is None
                        or mtime > self.last_mtime[src_path]
                    ):
                        logging.info(f"File update detected: {src_path}")
                        if self.is_stable(src_path):
                            os.makedirs(os.path.dirname(dst_path), exist_ok=True)
                            try:
                                shutil.copy2(src_path, dst_path)
                                self.last_mtime[src_path] = mtime
                                logging.info(
                                    f"Backup completed: {src_path} -> {dst_path}"
                                )
                            except Exception as e:
                                logging.error(
                                    f"Backup failed: {src_path} -> {dst_path} - {e}"
                                )
                        else:
                            logging.info(
                                f"File is still being written, skipping: {src_path}"
                            )

    def start(self, **kwargs):
        logging.info("Mirror Folders Service started")
        while True:
            self.mirror()
            time.sleep(self.mirror_interval)

    def stop(self, **kwargs):
        pass

    def restart(self, **kwargs):
        pass



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import unitorch.cli.models.peft.diffusers.modeling_stable
import unitorch.cli.models.peft.diffusers.modeling_stable_xl
import unitorch.cli.models.peft.diffusers.modeling_stable_3
import unitorch.cli.models.peft.diffusers.modeling_stable_flux
import unitorch.cli.models.peft.diffusers.modeling_qwen_image
import unitorch.cli.models.peft.diffusers.modeling_wan



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import io
import torch
import gc
import gradio as gr
from PIL import Image, ImageOps
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.utils import nested_dict_value
from unitorch.cli import CoreConfigureParser, GenericWebUI
from unitorch.cli import register_webui
from unitorch.cli.models.diffusers import (
    pretrained_stable_infos,
    pretrained_stable_extensions_infos,
)
from unitorch.cli.pipelines.stable.inpainting import StableForImageInpaintingPipeline

from unitorch.cli.webuis import (
    supported_scheduler_names,
    matched_pretrained_names,
    create_element,
    create_accordion,
    create_row,
    create_column,
    create_group,
    create_tab,
    create_tabs,
    create_blocks,
    create_pretrain_layout,
    create_lora_layout,
)
from unitorch.cli.webuis import SimpleWebUI


class StableImageInpaintingWebUI(SimpleWebUI):
    pretrained_names = list(pretrained_stable_infos.keys())
    supported_pretrained_names = matched_pretrained_names(
        pretrained_names, ["^stable-v1.5-", "^stable-v2-", "^stable-v2.1"]
    )
    pretrained_extension_names = list(pretrained_stable_extensions_infos.keys())
    supported_lora_names = matched_pretrained_names(
        pretrained_extension_names,
        ["^stable-v1.5-lora-", "^stable-v2-lora-", "^stable-v2.1-lora-"],
    )
    supported_schedulers = supported_scheduler_names

    def __init__(self, config: CoreConfigureParser):
        self._pipe = None
        self._status = "Stopped" if self._pipe is None else "Running"
        if len(self.supported_pretrained_names) == 0:
            raise ValueError("No supported pretrained models found.")
        self._name = self.supported_pretrained_names[0]

        # create elements
        pretrain_layout_group = create_pretrain_layout(
            self.supported_pretrained_names, self._name
        )
        name, status, start, stop, pretrain_layout = (
            pretrain_layout_group.name,
            pretrain_layout_group.status,
            pretrain_layout_group.start,
            pretrain_layout_group.stop,
            pretrain_layout_group.layout,
        )

        prompt = create_element(
            "text", "Input Prompt", lines=3, placeholder="Prompt", show_label=False
        )
        negative_prompt = create_element(
            "text",
            "Input Negative Prompt",
            lines=3,
            placeholder="Negative Prompt",
            show_label=False,
        )
        scheduler = create_element(
            "dropdown",
            "Sampling Method",
            values=self.supported_schedulers,
            default=self.supported_schedulers[0],
        )
        steps = create_element(
            "slider", "Diffusion Steps", min_value=1, max_value=100, step=1, default=25
        )
        height = create_element(
            "slider", "Image Height", min_value=1, max_value=2048, step=1, default=1024
        )
        width = create_element(
            "slider", "Image Width", min_value=1, max_value=2048, step=1, default=1024
        )
        image = create_element("image_editor", "Input Image")
        mask_image = create_element("image", "Input Image Mask")

        guidance_scale = create_element(
            "slider", "Guidance Scale", min_value=0, max_value=50, step=0.1, default=7.5
        )
        strength = create_element(
            "slider", "Strength", min_value=0, max_value=1, step=0.01, default=0.8
        )

        seed = create_element(
            "slider", "Seed", min_value=0, max_value=9999, step=1, default=42
        )

        ## extensions

        self.num_loras = 5
        lora_layout_group = create_lora_layout(
            self.supported_lora_names, num_loras=self.num_loras
        )
        loras = lora_layout_group.loras
        lora_layout = lora_layout_group.layout
        lora_params = []
        for lora in loras:
            lora_params += [
                lora.checkpoint,
                lora.weight,
                lora.alpha,
                lora.url,
                lora.file,
            ]

        generate = create_element("button", "Generate", variant="primary", scale=2)
        output_image = create_element("image", "Output Image")

        # create layouts
        top1 = create_column(pretrain_layout)
        top2 = create_row(
            create_column(prompt, negative_prompt, scale=4),
            create_column(generate),
        )
        left_generation = create_tab(
            create_row(image, mask_image),
            create_row(scheduler, steps),
            create_row(height, width),
            create_row(guidance_scale, strength),
            create_row(seed),
            name="Generation",
        )
        left_extension = create_tab(
            create_row(lora_layout),
            name="Extensions",
        )
        left_settings = create_tab(
            name="Settings",
        )

        left = create_tabs(left_generation, left_extension, left_settings)
        right = create_column(output_image)
        iface = create_blocks(top1, top2, create_row(left, right))

        # create events
        iface.__enter__()

        start.click(fn=self.start, inputs=[name], outputs=[status], trigger_mode="once")
        stop.click(fn=self.stop, outputs=[status], trigger_mode="once")

        image.change(fn=self.composite_images, inputs=[image], outputs=[mask_image])

        for lora in loras:
            lora.checkpoint.change(
                fn=lambda x: nested_dict_value(
                    pretrained_stable_extensions_infos, x, "text"
                ),
                inputs=[lora.checkpoint],
                outputs=[lora.text],
            )

        generate.click(
            fn=self.generate,
            inputs=[
                prompt,
                image,
                mask_image,
                negative_prompt,
                height,
                width,
                guidance_scale,
                strength,
                steps,
                seed,
                scheduler,
                *lora_params,
            ],
            outputs=[output_image],
            trigger_mode="once",
        )
        image.change(
            lambda x: (
                x["background"].size
                if nested_dict_value(x, "background") is not None
                else (1024, 1024)
            ),
            inputs=[image],
            outputs=[width, height],
        )
        iface.load(
            fn=lambda: [gr.update(value=self._name), gr.update(value=self._status)],
            outputs=[name, status],
        )

        iface.__exit__()

        super().__init__(config, iname="Inpainting", iface=iface)

    def start(self, pretrained_name, **kwargs):
        if self._name == pretrained_name and self._status == "Running":
            return self._status
        if self._status == "Running":
            self.stop()
        self._name = pretrained_name
        self._pipe = StableForImageInpaintingPipeline.from_core_configure(
            self._config,
            pretrained_name=pretrained_name,
        )
        self._status = "Running"
        return self._status

    def stop(self, **kwargs):
        self._pipe.to("cpu")
        del self._pipe
        gc.collect()
        torch.cuda.empty_cache()
        self._pipe = None
        self._status = "Stopped" if self._pipe is None else "Running"
        return self._status

    def composite_images(self, images):
        if images is None:
            return None
        layers = images["layers"]
        if len(layers) == 0:
            return None
        image = layers[0]
        for i in range(1, len(layers)):
            image = Image.alpha_composite(image, layers[i])
        image = image.convert("L")
        image = image.point(lambda p: p < 5 and 255)
        image = ImageOps.invert(image)
        return image

    def generate(
        self,
        text: str,
        image: Image.Image,
        mask_image: Image.Image,
        negative_text: str,
        height: int,
        width: int,
        guidance_scale: float,
        strength: float,
        num_timesteps: int,
        seed: int,
        scheduler: str,
        *params,
    ):
        assert self._pipe is not None

        lora_params = params

        lora_checkpoints = lora_params[::5]
        lora_weights = lora_params[1::5]
        lora_alphas = lora_params[2::5]
        lora_urls = lora_params[3::5]
        lora_files = lora_params[4::5]
        image = self._pipe(
            text,
            image["background"],
            mask_image,
            negative_text,
            width=width,
            height=height,
            guidance_scale=guidance_scale,
            strength=strength,
            num_timesteps=num_timesteps,
            seed=seed,
            scheduler=scheduler,
            lora_checkpoints=lora_checkpoints,
            lora_weights=lora_weights,
            lora_alphas=lora_alphas,
            lora_urls=lora_urls,
            lora_files=lora_files,
        )
        return image



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import random
from functools import partial
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union

import numpy as np
import torch

from transformers import BloomTokenizerFast, AddedToken
from unitorch.utils import (
    pop_value,
    truncate_sequence_pair,
    read_json_file,
    get_added_token,
)
from unitorch.models import (
    HfTextClassificationProcessor,
    HfTextGenerationProcessor,
    GenericOutputs,
)


class BloomProcessor(HfTextClassificationProcessor, HfTextGenerationProcessor):
    """
    Processor for the Bloom model that combines text classification and text generation functionality.
    """

    def __init__(
        self,
        tokenizer_file: str,
        tokenizer_config: Optional[str] = None,
        special_tokens_map: Optional[str] = None,
        chat_template: Optional[str] = None,
        max_seq_length: Optional[int] = 128,
        max_gen_seq_length: Optional[int] = 48,
    ):
        """
        Initializes a new instance of the BloomProcessor.

        Args:
            tokenizer_file (str): The path to the tokenizer file.
            max_seq_length (Optional[int]): The maximum sequence length for classification. Defaults to 128.
            max_gen_seq_length (Optional[int]): The maximum sequence length for generation. Defaults to 48.
        """
        tokenizer_config = read_json_file(tokenizer_config) if tokenizer_config else {}
        special_tokens_map = (
            read_json_file(special_tokens_map) if special_tokens_map else {}
        )
        added_tokens_decoder = tokenizer_config.pop("added_tokens_decoder", {})
        tokenizer_config = {
            k: (
                get_added_token(v)
                if isinstance(v, dict) and v.get("__type") == "AddedToken"
                else v
            )
            for k, v in tokenizer_config.items()
        }

        tokenizer = BloomTokenizerFast(
            tokenizer_file=tokenizer_file,
            **tokenizer_config,
        )

        for idx, spec in added_tokens_decoder.items():
            token = spec["content"]
            tokenizer.added_tokens_decoder[idx] = get_added_token(spec)
            tokenizer.added_tokens_encoder[token] = idx

        special_tokens = {}
        for name, spec in special_tokens_map.items():
            special_tokens[name] = get_added_token(spec)
        tokenizer.add_special_tokens(special_tokens)

        if chat_template:
            tokenizer.chat_template = read_json_file(chat_template)["chat_template"]
        tokenizer.cls_token = tokenizer.bos_token
        tokenizer.sep_token = tokenizer.eos_token
        tokenizer.cls_token_id = tokenizer.bos_token_id
        tokenizer.sep_token_id = tokenizer.eos_token_id
        HfTextClassificationProcessor.__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
        )
        HfTextGenerationProcessor.__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )

    def chat_template(
        self,
        messages: List[Dict[str, Any]],
    ):
        text = self.tokenizer.apply_chat_template(
            messages,
            tokenize=False,
            add_generation_prompt=True,
        )
        return text

    def classification(
        self,
        text: str,
        text_pair: Optional[str] = None,
        max_seq_length: Optional[int] = None,
    ) -> GenericOutputs:
        """
        Preprocesses text for classification.

        Args:
            text (str): The input text to classify.
            text_pair (Optional[str]): The second input text for sequence classification. Defaults to None.
            max_seq_length (Optional[int]): The maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: The processed input IDs and attention mask tensors.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )

        tokens = self.tokenizer.tokenize(str(text))
        if text_pair is None:
            tokens = tokens[:max_seq_length]
            input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
        else:
            tokens_pair = self.tokenizer.tokenize(str(text_pair))
            truncate_sequence_pair(tokens, tokens_pair, max_seq_length)
            tokens = tokens + tokens_pair
            input_ids = self.tokenizer.convert_tokens_to_ids(tokens)

        padding = [0] * (max_seq_length - len(input_ids))
        attention_mask = [0] * len(padding) + [1] * len(input_ids)
        input_ids = len(padding) * [self.pad_token_id] + input_ids

        assert len(input_ids) == max_seq_length
        assert len(attention_mask) == max_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
        )

    def generation_inputs(
        self,
        text: str,
        max_seq_length: Optional[int] = None,
    ) -> GenericOutputs:
        """
        Preprocesses text as generation inputs.

        Args:
            text (str): The input text for generation.
            max_seq_length (Optional[int]): The maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: The processed input IDs tensor.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )
        tokens = self.tokenizer.tokenize(str(text))[-max_seq_length:]
        padding = [self.pad_token] * (max_seq_length - len(tokens))
        tokens = padding + tokens
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)

        assert len(input_ids) == max_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
        )

    def generation_labels(
        self,
        text: str,
        max_gen_seq_length: Optional[int] = None,
    ) -> GenericOutputs:
        """
        Preprocesses text as generation labels.

        Args:
            text (str): The input text for generation labels.
            max_gen_seq_length (Optional[int]): The maximum generation sequence length. Defaults to None.

        Returns:
            GenericOutputs: The processed input IDs and attention mask tensors.
        """
        max_gen_seq_length = pop_value(
            max_gen_seq_length,
            self.max_gen_seq_length,
        )
        tokens = self.tokenizer.tokenize(str(text))[: max_gen_seq_length - 1] + [
            self.eos_token
        ]
        padding = [self.pad_token] * (max_gen_seq_length - len(tokens))
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
        attention_mask = [1] * len(input_ids)

        padding = [0] * (max_gen_seq_length - len(input_ids))
        input_ids += [self.pad_token_id] * len(padding)
        attention_mask += padding

        assert len(input_ids) == max_gen_seq_length
        assert len(attention_mask) == max_gen_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
        )

    def generation(
        self,
        text: str,
        text_pair: str,
        max_seq_length: Optional[int] = None,
        max_gen_seq_length: Optional[int] = None,
    ) -> GenericOutputs:
        """
        Preprocesses text for generation.

        Args:
            text (str): The input text for generation.
            text_pair (str): The second input text for generation.
            max_seq_length (Optional[int]): The maximum sequence length for classification. Defaults to None.
            max_gen_seq_length (Optional[int]): The maximum generation sequence length. Defaults to None.

        Returns:
            GenericOutputs: The processed input IDs and attention mask tensors.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )
        max_gen_seq_length = pop_value(
            max_gen_seq_length,
            self.max_gen_seq_length,
        )

        tokens = self.tokenizer.tokenize(str(text))[-max_seq_length:]
        tokens_pair = self.tokenizer.tokenize(str(text_pair))[
            : max_gen_seq_length - 1
        ] + [self.eos_token]
        padding_a = [self.pad_token] * (max_seq_length - len(tokens))
        padding_b = [self.pad_token] * (max_gen_seq_length - len(tokens_pair))
        attention_mask = (
            [0] * len(padding_a)
            + [1] * (len(tokens) + len(tokens_pair))
            + [0] * len(padding_b)
        )
        _tokens = padding_a + tokens + tokens_pair + padding_b
        input_ids = self.tokenizer.convert_tokens_to_ids(_tokens)

        tokens_label = tokens_pair + [self.pad_token] * (
            max_gen_seq_length - len(tokens_pair) + 1
        )
        input_ids_label = self.tokenizer.convert_tokens_to_ids(tokens_label)
        input_ids_label = [0] * (max_seq_length - 1) + input_ids_label
        attention_mask_label = [1] * len(tokens_pair) + [0] * (
            max_gen_seq_length - len(tokens_pair) + 1
        )
        attention_mask_label = [0] * (max_seq_length - 1) + attention_mask_label

        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
            input_ids_label=torch.tensor(input_ids_label, dtype=torch.long),
            attention_mask_label=torch.tensor(attention_mask_label, dtype=torch.long),
        )

    def messages_generation(
        self,
        messages: List[Dict[str, Any]],
        max_seq_length: Optional[int] = None,
    ) -> GenericOutputs:
        """
        Preprocesses messages for generation.

        Args:
            messages (List[Dict[str, Any]]): The list of messages to process.
            max_seq_length (Optional[int]): The maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: The processed input IDs tensor.
        """
        while messages and messages[-1]["role"] != "assistant":
            messages.pop()

        text = self.chat_template(messages[:-1])
        text_pair = self.chat_template(messages[-1:])
        outputs = self.generation(
            text=text,
            text_pair=text_pair,
            max_seq_length=max_seq_length,
        )
        return GenericOutputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            input_ids_label=outputs.input_ids_label,
            attention_mask_label=outputs.attention_mask_label,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import unitorch.cli.fastapis.stable_xl.text2image
import unitorch.cli.fastapis.stable_xl.image2image
import unitorch.cli.fastapis.stable_xl.inpainting



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from unitorch.utils import pop_value, nested_dict_value
from unitorch.models.diffusers import QWenImageProcessor as _QWenImageProcessor
from unitorch.cli import (
    cached_path,
    add_default_section_for_init,
    add_default_section_for_function,
    register_process,
)
from unitorch.cli.models import (
    TensorsInputs,
)
from unitorch.cli.models.diffusers import pretrained_stable_infos


class QWenImageProcessor(_QWenImageProcessor):
    def __init__(
        self,
        vocab_path: str,
        merge_path: str,
        vision_config_path: Optional[str] = None,
        vae_config_path: Optional[str] = None,
        tokenizer_config: Optional[str] = None,
        special_tokens_map: Optional[str] = None,
        max_seq_length: Optional[int] = 12800,
        image_size: Optional[Tuple[int, int]] = None,
        center_crop: Optional[bool] = False,
        random_flip: Optional[bool] = False,
    ):
        super().__init__(
            vocab_path=vocab_path,
            merge_path=merge_path,
            vision_config_path=vision_config_path,
            vae_config_path=vae_config_path,
            tokenizer_config=tokenizer_config,
            special_tokens_map=special_tokens_map,
            max_seq_length=max_seq_length,
            image_size=image_size,
            center_crop=center_crop,
            random_flip=random_flip,
        )

    @classmethod
    @add_default_section_for_init("core/process/diffusion/qwen_image")
    def from_core_configure(cls, config, **kwargs):
        config.set_default_section("core/process/diffusion/qwen_image")
        pretrained_name = config.getoption("pretrained_name", "qwen-image")
        pretrained_infos = nested_dict_value(pretrained_stable_infos, pretrained_name)

        vocab_path = config.getoption("vocab_path", None)
        vocab_path = pop_value(
            vocab_path,
            nested_dict_value(pretrained_infos, "text", "vocab"),
        )
        vocab_path = cached_path(vocab_path)

        merge_path = config.getoption("merge_path", None)
        merge_path = pop_value(
            merge_path,
            nested_dict_value(pretrained_infos, "text", "merge"),
        )
        merge_path = cached_path(merge_path)

        tokenizer_config = config.getoption("tokenizer_config", None)
        tokenizer_config = pop_value(
            tokenizer_config,
            nested_dict_value(pretrained_infos, "text", "tokenizer_config"),
            check_none=False,
        )
        tokenizer_config = (
            cached_path(tokenizer_config) if tokenizer_config is not None else None
        )

        special_tokens_map = config.getoption("special_tokens_map", None)
        special_tokens_map = pop_value(
            special_tokens_map,
            nested_dict_value(pretrained_infos, "text", "special_tokens_map"),
            check_none=False,
        )
        special_tokens_map = (
            cached_path(special_tokens_map) if special_tokens_map is not None else None
        )

        vision_config_path = config.getoption("vision_config_path", None)
        vision_config_path = pop_value(
            vision_config_path,
            nested_dict_value(pretrained_infos, "vision_config"),
            check_none=False,
        )
        if vision_config_path is not None:
            vision_config_path = cached_path(vision_config_path)

        vae_config_path = config.getoption("vae_config_path", None)
        vae_config_path = pop_value(
            vae_config_path,
            nested_dict_value(pretrained_infos, "vae", "config"),
        )
        vae_config_path = cached_path(vae_config_path)
        return {
            "vocab_path": vocab_path,
            "merge_path": merge_path,
            "vision_config_path": vision_config_path,
            "vae_config_path": vae_config_path,
            "tokenizer_config": tokenizer_config,
            "special_tokens_map": special_tokens_map,
        }

    @register_process("core/process/diffusion/qwen_image/text2image")
    def _text2image(
        self,
        prompt: str,
        image: Union[Image.Image, str],
        max_seq_length: Optional[int] = None,
    ):
        outputs = super().text2image(
            prompt=prompt,
            image=image,
            max_seq_length=max_seq_length,
        )
        return TensorsInputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            pixel_values=outputs.pixel_values,
        )

    @register_process("core/process/diffusion/qwen_image/text2image/inputs")
    def _text2image_inputs(
        self,
        prompt: str,
        negative_prompt: Optional[str] = "",
        max_seq_length: Optional[int] = None,
    ):
        outputs = super().text2image_inputs(
            prompt=prompt,
            negative_prompt=negative_prompt,
            max_seq_length=max_seq_length,
        )
        return TensorsInputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            negative_input_ids=outputs.negative_input_ids,
            negative_attention_mask=outputs.negative_attention_mask,
        )

    @register_process("core/process/diffusion/qwen_image/editing/inputs")
    def _editing_inputs(
        self,
        prompt: str,
        refer_image: Union[Image.Image, str],
        negative_prompt: Optional[str] = "",
        max_seq_length: Optional[int] = None,
    ):
        outputs = super().editing_inputs(
            prompt=prompt,
            refer_image=refer_image,
            negative_prompt=negative_prompt,
            max_seq_length=max_seq_length,
        )

        return TensorsInputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            refer_pixel_values=outputs.refer_pixel_values,
            refer_image_grid_thw=outputs.refer_image_grid_thw,
            refer_vae_pixel_values=outputs.refer_vae_pixel_values,
            negative_input_ids=outputs.negative_input_ids,
            negative_attention_mask=outputs.negative_attention_mask,
        )

    @register_process("core/process/diffusion/qwen_image/editing")
    def _editing(
        self,
        prompt: str,
        refer_image: Union[Image.Image, str],
        image: Union[Image.Image, str],
        max_seq_length: Optional[int] = None,
    ):
        outputs = super().editing(
            prompt=prompt,
            refer_image=refer_image,
            image=image,
            max_seq_length=max_seq_length,
        )
        return TensorsInputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            pixel_values=outputs.pixel_values,
            refer_image_grid_thw=outputs.refer_image_grid_thw,
            refer_pixel_values=outputs.refer_pixel_values,
            refer_vae_pixel_values=outputs.refer_vae_pixel_values,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import unitorch.cli.fastapis.stable.text2image
import unitorch.cli.fastapis.stable.image2image
import unitorch.cli.fastapis.stable.inpainting
import unitorch.cli.fastapis.stable.resolution
import unitorch.cli.fastapis.stable.image2video
import unitorch.cli.fastapis.stable.interrogator



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

pretrained_peft_infos = {}

from unitorch.utils import is_diffusers_available

import unitorch.cli.models.peft.modeling_bloom
import unitorch.cli.models.peft.modeling_clip
import unitorch.cli.models.peft.modeling_llama
import unitorch.cli.models.peft.modeling_llava
import unitorch.cli.models.peft.modeling_mistral
import unitorch.cli.models.peft.modeling_qwen
import unitorch.cli.models.peft.modeling_qwen_vl

if is_diffusers_available():
    import unitorch.cli.models.peft.diffusers



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.utils import is_diffusers_available
import unitorch.cli.fastapis.info
import unitorch.cli.fastapis.bria
import unitorch.cli.fastapis.qwen
import unitorch.cli.fastapis.qwen_vl
import unitorch.cli.fastapis.llava

if is_diffusers_available():
    import unitorch.cli.fastapis.stable
    import unitorch.cli.fastapis.stable_xl
    import unitorch.cli.fastapis.stable_3
    import unitorch.cli.fastapis.stable_flux
    import unitorch.cli.fastapis.qwen_image
    import unitorch.cli.fastapis.wan



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import json
import random
from functools import partial
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from PIL import Image
import numpy as np
import torch
from torchvision.transforms import (
    Resize,
    CenterCrop,
    RandomCrop,
    Lambda,
    ToTensor,
    Normalize,
    Compose,
    RandomHorizontalFlip,
)
from transformers import (
    Qwen2VLImageProcessor,
    Qwen2Tokenizer,
    Qwen2TokenizerFast,
    Qwen2_5_VLProcessor,
)
from diffusers.image_processor import VaeImageProcessor

from unitorch.utils import (
    pop_value,
    truncate_sequence_pair,
    read_json_file,
    get_added_token,
)
from unitorch.models import (
    HfTextClassificationProcessor,
    HfTextGenerationProcessor,
    HfImageClassificationProcessor,
    GenericOutputs,
)


class QWenImageProcessor(HfTextClassificationProcessor):
    def __init__(
        self,
        vocab_path: str,
        merge_path: str,
        vision_config_path: Optional[str] = None,
        vae_config_path: Optional[str] = None,
        tokenizer_config: Optional[str] = None,
        special_tokens_map: Optional[str] = None,
        max_seq_length: Optional[int] = 12800,
        image_size: Optional[Tuple[int, int]] = None,
        center_crop: Optional[bool] = False,
        random_flip: Optional[bool] = False,
    ):
        """
        Initializes the ClipProcessor.

        Args:
            vocab_path (str): The path to the vocabulary file.
            merge_path (str): The path to the merge file.
            max_seq_length (int, optional): The maximum sequence length for text inputs. Defaults to 262144.
        """
        tokenizer_config = read_json_file(tokenizer_config) if tokenizer_config else {}
        special_tokens_map = (
            read_json_file(special_tokens_map) if special_tokens_map else {}
        )
        added_tokens_decoder = tokenizer_config.pop("added_tokens_decoder", {})
        tokenizer_config = {
            k: (
                get_added_token(v)
                if isinstance(v, dict) and v.get("__type") == "AddedToken"
                else v
            )
            for k, v in tokenizer_config.items()
        }
        tokenizer = Qwen2TokenizerFast(
            vocab_file=vocab_path,
            merges_file=merge_path,
            **tokenizer_config,
        )
        for idx, spec in added_tokens_decoder.items():
            token = spec["content"]
            tokenizer.added_tokens_decoder[idx] = get_added_token(spec)
            tokenizer.added_tokens_encoder[token] = idx

        special_tokens = {}
        for name, spec in special_tokens_map.items():
            if not isinstance(spec, dict or str):
                continue
            special_tokens[name] = get_added_token(spec)
        tokenizer.add_special_tokens(special_tokens)

        tokenizer.cls_token = tokenizer.bos_token
        tokenizer.sep_token = tokenizer.eos_token

        super().__init__(
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
        )

        self.image_token = (
            "<|image_pad|>"
            if not hasattr(tokenizer, "image_token")
            else tokenizer.image_token
        )
        self.video_token = (
            "<|video_pad|>"
            if not hasattr(tokenizer, "video_token")
            else tokenizer.video_token
        )
        self.image_token_id = (
            tokenizer.image_token_id
            if getattr(tokenizer, "image_token_id", None)
            else tokenizer.convert_tokens_to_ids(self.image_token)
        )
        self.video_token_id = (
            tokenizer.video_token_id
            if getattr(tokenizer, "video_token_id", None)
            else tokenizer.convert_tokens_to_ids(self.video_token)
        )

        if vision_config_path is not None:
            self.refer_vision_processor = Qwen2VLImageProcessor.from_json_file(
                vision_config_path
            )
        else:
            self.refer_vision_processor = None

        if image_size is not None:
            self.image_size = (
                image_size
                if isinstance(image_size, tuple)
                else (image_size, image_size)
            )
        else:
            self.image_size = None

        if self.image_size is not None:
            self.vision_processor = Compose(
                [
                    Resize((self.image_size[1], self.image_size[0])),
                    (
                        CenterCrop((self.image_size[1], self.image_size[0]))
                        if center_crop
                        else RandomCrop((self.image_size[1], self.image_size[0]))
                    ),
                    RandomHorizontalFlip() if random_flip else Lambda(lambda x: x),
                    ToTensor(),
                    Normalize([0.5], [0.5]),
                ]
            )
        else:
            self.vision_processor = Compose(
                [
                    RandomHorizontalFlip() if random_flip else Lambda(lambda x: x),
                    ToTensor(),
                    Normalize([0.5], [0.5]),
                ]
            )

        if vae_config_path is not None:
            vae_config_dict = json.load(open(vae_config_path))
            vae_scale_factor = 2 ** (
                len(vae_config_dict.get("block_out_channels", [])) - 1
            )
            self.vae_image_processor = VaeImageProcessor(
                vae_scale_factor=vae_scale_factor * 2
            )
        else:
            self.vae_image_processor = None

        self.prompt_template = "<|im_start|>system\nDescribe the image by detailing the color, shape, size, texture, quantity, text, spatial relationships of the objects and background:<|im_end|>\n<|im_start|>user\n{}<|im_end|>\n<|im_start|>assistant\n"
        self.prompt_template_editing = "<|im_start|>system\nDescribe the key features of the input image (color, shape, size, texture, objects, background), then explain how the user's text instruction should alter or modify the image. Generate a new image that meets the user's requirements while maintaining consistency with the original input where appropriate.<|im_end|>\n<|im_start|>user\n<|vision_start|><|image_pad|><|vision_end|>{}<|im_end|>\n<|im_start|>assistant\n"
        self.prompt_start_index = 34
        self.prompt_editing_start_index = 64

    def processing_image(
        self,
        image: Union[Image.Image, str],
    ):
        """
        Process images for classification.

        Args:
            images (Image.Image, str, List[Image.Image], List[str]): Input image or list of images.

        Returns:
            GenericOutputs: Processed outputs.
        """
        if isinstance(image, str):
            image = Image.open(image)
        outputs = self.refer_vision_processor(images=[image], return_tensors="pt")
        return outputs

    def text2image_inputs(
        self,
        prompt: str,
        negative_prompt: Optional[str] = "",
        max_seq_length: Optional[int] = None,
    ):
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )
        max_seq_length = max_seq_length + self.prompt_start_index
        prompt = self.prompt_template.format(str(prompt))
        tokens = self.tokenizer.tokenize(prompt)
        tokens = tokens[:max_seq_length]
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)

        padding = [self.pad_token_id] * (max_seq_length - len(input_ids))
        attention_mask = [1] * len(input_ids) + [0] * len(padding)
        input_ids = input_ids + padding

        if negative_prompt is not None:
            negative_prompt = self.prompt_template.format(str(negative_prompt))
            negative_tokens = self.tokenizer.tokenize(negative_prompt)
            negative_tokens = negative_tokens[:max_seq_length]
            negative_input_ids = self.tokenizer.convert_tokens_to_ids(negative_tokens)
            negative_padding = [self.pad_token_id] * (
                max_seq_length - len(negative_input_ids)
            )
            negative_attention_mask = [1] * len(negative_input_ids) + [0] * len(
                negative_padding
            )
            negative_input_ids = negative_input_ids + negative_padding
        else:
            negative_input_ids = None
            negative_attention_mask = None

        assert len(input_ids) == max_seq_length
        assert len(attention_mask) == max_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
            negative_input_ids=(
                torch.tensor(negative_input_ids, dtype=torch.long)
                if negative_input_ids is not None
                else None
            ),
            negative_attention_mask=(
                torch.tensor(negative_attention_mask, dtype=torch.long)
                if negative_attention_mask is not None
                else None
            ),
        )

    def text2image(
        self,
        prompt: str,
        image: Union[Image.Image, str],
        max_seq_length: Optional[int] = None,
    ):
        prompt_outputs = self.text2image_inputs(
            prompt=prompt,
            negative_prompt=None,
            max_seq_length=max_seq_length,
        )
        pixel_values = self.vision_processor(image)

        return GenericOutputs(
            input_ids=prompt_outputs.input_ids,
            attention_mask=prompt_outputs.attention_mask,
            pixel_values=pixel_values,
        )

    def editing_inputs(
        self,
        prompt: str,
        refer_image: Union[Image.Image, str],
        negative_prompt: Optional[str] = "",
        max_seq_length: Optional[int] = None,
    ):
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )
        max_seq_length = max_seq_length + self.prompt_editing_start_index
        prompt = self.prompt_template_editing.format(str(prompt))
        image_inputs = self.processing_image(refer_image)
        image_index, image_merge_size = 0, self.refer_vision_processor.merge_size**2
        image_grid_thw = image_inputs["image_grid_thw"]
        while self.image_token in prompt:
            num_image_tokens = image_grid_thw[image_index].prod() // image_merge_size
            prompt = prompt.replace(
                self.image_token,
                "<|placeholder|>" * num_image_tokens,
                1,
            )
            image_index += 1
        prompt = prompt.replace("<|placeholder|>", self.image_token)

        tokens = self.tokenizer.tokenize(prompt)
        tokens = tokens[:max_seq_length]
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)

        padding = [self.pad_token_id] * (max_seq_length - len(input_ids))
        attention_mask = [1] * len(input_ids) + [0] * len(padding)
        input_ids = input_ids + padding

        if negative_prompt is not None:
            negative_prompt = self.prompt_template_editing.format(str(negative_prompt))
            image_index, image_merge_size = 0, self.refer_vision_processor.merge_size**2
            while self.image_token in negative_prompt:
                num_image_tokens = (
                    image_grid_thw[image_index].prod() // image_merge_size
                )
                negative_prompt = negative_prompt.replace(
                    self.image_token,
                    "<|placeholder|>" * num_image_tokens,
                    1,
                )
                image_index += 1
            negative_prompt = negative_prompt.replace(
                "<|placeholder|>", self.image_token
            )
            negative_tokens = self.tokenizer.tokenize(negative_prompt)
            negative_tokens = negative_tokens[:max_seq_length]
            negative_input_ids = self.tokenizer.convert_tokens_to_ids(negative_tokens)
            negative_padding = [self.pad_token_id] * (
                max_seq_length - len(negative_input_ids)
            )
            negative_attention_mask = [1] * len(negative_input_ids) + [0] * len(
                negative_padding
            )
            negative_input_ids = negative_input_ids + negative_padding
        else:
            negative_input_ids = None
            negative_attention_mask = None

        refer_vae_pixel_values = self.vae_image_processor.preprocess(refer_image)[0]

        assert len(input_ids) == max_seq_length
        assert len(attention_mask) == max_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
            negative_input_ids=(
                torch.tensor(negative_input_ids, dtype=torch.long)
                if negative_input_ids is not None
                else None
            ),
            negative_attention_mask=(
                torch.tensor(negative_attention_mask, dtype=torch.long)
                if negative_attention_mask is not None
                else None
            ),
            refer_image_grid_thw=torch.tensor(
                image_inputs["image_grid_thw"], dtype=torch.long
            ),
            refer_pixel_values=torch.tensor(image_inputs["pixel_values"]),
            refer_vae_pixel_values=torch.tensor(refer_vae_pixel_values),
        )

    def editing(
        self,
        prompt: str,
        refer_image: Union[Image.Image, str],
        image: Union[Image.Image, str],
        max_seq_length: Optional[int] = None,
    ):
        prompt_outputs = self.editing_inputs(
            prompt=prompt,
            refer_image=refer_image,
            negative_prompt=None,
            max_seq_length=max_seq_length,
        )
        pixel_values = self.vision_processor(image)

        return GenericOutputs(
            input_ids=prompt_outputs.input_ids,
            attention_mask=prompt_outputs.attention_mask,
            refer_image_grid_thw=prompt_outputs.refer_image_grid_thw,
            refer_pixel_values=prompt_outputs.refer_pixel_values,
            refer_vae_pixel_values=prompt_outputs.refer_vae_pixel_values,
            pixel_values=pixel_values,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import numpy as np
from PIL import Image, ImageFilter
from typing import Any, Dict, List, Optional, Tuple, Union
from unitorch.utils import is_opencv_available

import unitorch.cli.pipelines.stable_xl.text2image
import unitorch.cli.pipelines.stable_xl.image2image
import unitorch.cli.pipelines.stable_xl.inpainting



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
from PIL import Image
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from transformers import PreTrainedTokenizer
from transformers.image_utils import to_numpy_array, ChannelDimension
from transformers.image_transforms import to_channel_dimension_format
from transformers.image_processing_utils import BaseImageProcessor
from unitorch.models import GenericOutputs
from unitorch.utils import pop_value, truncate_sequence_pair


class HfTextGenerationProcessor:
    """
    Processor for text generation tasks.

    Args:
        tokenizer (PreTrainedTokenizer): The tokenizer to use for text generation.
        max_seq_length (int, optional): Maximum sequence length. Defaults to 128.
        max_gen_seq_length (int, optional): Maximum generated sequence length. Defaults to 48.
    """

    def __init__(
        self,
        tokenizer: PreTrainedTokenizer,
        max_seq_length: Optional[int] = 128,
        max_gen_seq_length: Optional[int] = 48,
    ):
        self.tokenizer = tokenizer
        self.max_seq_length = max_seq_length
        self.max_gen_seq_length = max_gen_seq_length
        self.pad_token = self.tokenizer.pad_token
        self.bos_token = self.tokenizer.bos_token
        self.eos_token = self.tokenizer.eos_token
        self.mask_token = self.tokenizer.mask_token
        self.pad_token_id = self.tokenizer.pad_token_id
        self.vocab_size = len(self.tokenizer.get_vocab())

    def generation_inputs(
        self,
        text: str,
        max_seq_length: Optional[int] = None,
    ):
        """
        Generate inputs for text generation.

        Args:
            text (str): The input text.
            max_seq_length (int, optional): Maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: The generated input tokens and attention mask.
        """
        max_seq_length = pop_value(max_seq_length, self.max_seq_length)
        tokens = self.tokenizer.tokenize(str(text))
        tokens = tokens[:max_seq_length]
        if self.bos_token is not None:
            tokens = tokens[: max_seq_length - 1]
            tokens = [self.bos_token] + tokens

        if self.eos_token is not None:
            tokens = tokens[: max_seq_length - 1]
            tokens = tokens + [self.eos_token]
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
        input_ids = input_ids[:max_seq_length]
        attention_mask = [1] * len(input_ids)

        padding = [0] * (max_seq_length - len(input_ids))
        input_ids += [self.pad_token_id] * len(padding)
        attention_mask += padding

        assert len(input_ids) == max_seq_length
        assert len(attention_mask) == max_seq_length

        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
        )

    def generation_labels(
        self,
        text: str,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Generate labels for text generation.

        Args:
            text (str): The input text.
            max_gen_seq_length (int, optional): Maximum generated sequence length. Defaults to None.

        Returns:
            GenericOutputs: The generated label tokens and attention mask.
        """
        max_gen_seq_length = pop_value(max_gen_seq_length, self.max_gen_seq_length)
        tokens = self.tokenizer.tokenize(str(text))
        tokens = tokens[:max_gen_seq_length]
        if self.bos_token is not None:
            tokens = tokens[: max_gen_seq_length - 1]
            tokens = [self.bos_token] + tokens
        if self.eos_token is not None:
            tokens = tokens[: max_gen_seq_length - 1]
            tokens = tokens + [self.eos_token]
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
        input_ids = input_ids[1:max_gen_seq_length]
        attention_mask = [1] * len(input_ids)

        padding = [0] * (max_gen_seq_length - len(input_ids))
        input_ids += [self.pad_token_id] * len(padding)
        attention_mask += padding

        assert len(input_ids) == max_gen_seq_length

        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
        )

    def generation(
        self,
        text: str,
        text_pair: str,
        max_seq_length: Optional[int] = None,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Generate inputs, labels, and tokens for text generation.

        Args:
            text (str): The input text.
            text_pair (str): The paired text.
            max_seq_length (int, optional): Maximum sequence length. Defaults to None.
            max_gen_seq_length (int, optional): Maximum generated sequence length. Defaults to None.

        Returns:
            GenericOutputs: The generated input tokens, attention masks, label tokens, and attention masks.
        """
        max_seq_length = pop_value(max_seq_length, self.max_seq_length)
        max_gen_seq_length = pop_value(max_gen_seq_length, self.max_gen_seq_length)

        tokens = self.generation_inputs(text, max_seq_length)
        tokens_pair = self.generation_inputs(text_pair, max_gen_seq_length)
        labels = self.generation_labels(text_pair, max_gen_seq_length)

        return GenericOutputs(
            input_ids=tokens.input_ids,
            attention_mask=tokens.attention_mask,
            input_ids_pair=tokens_pair.input_ids,
            attention_mask_pair=tokens_pair.attention_mask,
            input_ids_label=labels.input_ids,
            attention_mask_label=labels.attention_mask,
        )

    def detokenize(
        self,
        sequences: torch.Tensor,
        skip_special_tokens: Optional[bool] = True,
    ):
        """
        Detokenize the sequences.

        Args:
            sequences (torch.Tensor): The sequences to detokenize.
            skip_special_tokens (bool, optional): Whether to skip special tokens. Defaults to True.

        Returns:
            list: The detokenized sequences.
        """
        if sequences.dim() == 3:
            _, num_return_sequences, sequences_length = sequences.size()
            sequences = sequences.reshape(-1, sequences_length).clamp_max(
                self.vocab_size
            )
            sequences = sequences.clamp_min(0)
            sequences[sequences == self.vocab_size] = self.pad_token_id
            decode_tokens = self.tokenizer.batch_decode(
                sequences,
                skip_special_tokens=skip_special_tokens,
            )
            decode_tokens = [
                decode_tokens[i : i + num_return_sequences]
                for i in range(0, len(decode_tokens), num_return_sequences)
            ]
        elif sequences.dim() == 2:
            sequences = sequences.clamp_min(0).clamp_max(self.vocab_size)
            sequences[sequences == self.vocab_size] = self.pad_token_id
            decode_tokens = self.tokenizer.batch_decode(
                sequences,
                skip_special_tokens=skip_special_tokens,
            )
        else:
            raise ValueError(f"Can't decode the tensor with shape {sequences.shape}")

        return decode_tokens


class HfLlmProcessor:
    """
    Processor for LLM tasks.

    Args:
        tokenizer (PreTrainedTokenizer): The tokenizer to use for LLM tasks.
        max_seq_length (int, optional): Maximum sequence length. Defaults to 2048.
    """

    def __init__(
        self,
        tokenizer: PreTrainedTokenizer,
        max_seq_length: Optional[int] = 2048,
        max_gen_seq_length: Optional[int] = 512,
    ):
        self.tokenizer = tokenizer
        self.max_seq_length = max_seq_length
        self.max_gen_seq_length = max_gen_seq_length
        self.pad_token = self.tokenizer.pad_token
        self.bos_token = self.tokenizer.bos_token
        self.eos_token = self.tokenizer.eos_token
        self.mask_token = self.tokenizer.mask_token
        self.pad_token_id = self.tokenizer.pad_token_id

    def chat_template(
        self,
        messages: List[Dict[str, Any]],
    ):
        text = self.tokenizer.apply_chat_template(
            messages,
            tokenize=False,
            add_generation_prompt=True,
        )
        return text

    def classification(
        self,
        text: str,
        text_pair: Optional[str] = None,
        max_seq_length: Optional[int] = None,
    ):
        """
        Process text for classification.

        Args:
            text (str): Input text.
            text_pair (str, optional): Input text pair. Defaults to None.
            max_seq_length (int, optional): Maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids and attention_mask tensors.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )

        tokens = self.tokenizer.tokenize(str(text))
        if text_pair is None:
            tokens = tokens[:max_seq_length]
            input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
        else:
            tokens_pair = self.tokenizer.tokenize(str(text_pair))
            truncate_sequence_pair(tokens, tokens_pair, max_seq_length)
            tokens = tokens + tokens_pair
            input_ids = self.tokenizer.convert_tokens_to_ids(tokens)

        padding = [0] * (max_seq_length - len(input_ids))
        attention_mask = [0] * len(padding) + [1] * len(input_ids)
        input_ids = len(padding) * [self.pad_token_id] + input_ids

        assert len(input_ids) == max_seq_length
        assert len(attention_mask) == max_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
        )

    def generation_inputs(
        self,
        text: str,
        max_seq_length: Optional[int] = None,
    ):
        """
        Process text for generation inputs.

        Args:
            text (str): Input text.
            max_seq_length (int, optional): Maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids tensor.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )
        if self.bos_token is not None:
            tokens = [self.bos_token] + self.tokenizer.tokenize(str(text))[
                -(max_seq_length - 1) :
            ]
        else:
            tokens = self.tokenizer.tokenize(str(text))[-max_seq_length:]
        padding = [self.pad_token] * (max_seq_length - len(tokens))
        attention_mask = [0] * len(padding) + [1] * len(tokens)
        tokens = padding + tokens
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
        assert len(input_ids) == max_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
        )

    def generation_labels(self, text, max_gen_seq_length=None):
        """
        Process text for generation labels.

        Args:
            text (str): Input text.
            max_gen_seq_length (int, optional): Maximum generation sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids and attention_mask tensors.
        """
        max_gen_seq_length = pop_value(
            max_gen_seq_length,
            self.max_gen_seq_length,
        )
        tokens = self.tokenizer.tokenize(str(text))[: max_gen_seq_length - 1] + [
            self.eos_token
        ]
        padding = [self.pad_token] * (max_gen_seq_length - len(tokens))
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
        attention_mask = [1] * len(input_ids)

        padding = [0] * (max_gen_seq_length - len(input_ids))
        input_ids += [self.pad_token_id] * len(padding)
        attention_mask += padding

        assert len(input_ids) == max_gen_seq_length
        assert len(attention_mask) == max_gen_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
        )

    def generation(
        self,
        text: str,
        text_pair: str,
        max_seq_length: Optional[int] = None,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Process text for generation.

        Args:
            text (str): Input text.
            text_pair (str): Input text pair.
            max_seq_length (int, optional): Maximum sequence length. Defaults to None.
            max_gen_seq_length (int, optional): Maximum generation sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids, attention_mask, input_ids_label, and attention_mask_label tensors.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )
        max_gen_seq_length = pop_value(
            max_gen_seq_length,
            self.max_gen_seq_length,
        )

        if self.bos_token is not None:
            tokens = [self.bos_token] + self.tokenizer.tokenize(str(text))[
                -(max_seq_length - 1) :
            ]
        else:
            tokens = self.tokenizer.tokenize(str(text))[-max_seq_length:]
        tokens_pair = self.tokenizer.tokenize(str(text_pair))[
            : max_gen_seq_length - 1
        ] + [self.eos_token]
        padding_a = [self.pad_token] * (max_seq_length - len(tokens))
        padding_b = [self.pad_token] * (max_gen_seq_length - len(tokens_pair))
        attention_mask = (
            [0] * len(padding_a)
            + [1] * (len(tokens) + len(tokens_pair))
            + [0] * len(padding_b)
        )
        _tokens = padding_a + tokens + tokens_pair + padding_b
        input_ids = self.tokenizer.convert_tokens_to_ids(_tokens)

        tokens_label = tokens_pair + [self.pad_token] * (
            max_gen_seq_length - len(tokens_pair) + 1
        )
        input_ids_label = self.tokenizer.convert_tokens_to_ids(tokens_label)
        input_ids_label = [0] * (max_seq_length - 1) + input_ids_label
        attention_mask_label = [1] * len(tokens_pair) + [0] * (
            max_gen_seq_length - len(tokens_pair) + 1
        )
        attention_mask_label = [0] * (max_seq_length - 1) + attention_mask_label

        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
            input_ids_label=torch.tensor(input_ids_label, dtype=torch.long),
            attention_mask_label=torch.tensor(attention_mask_label, dtype=torch.long),
        )

    def messages_generation(
        self,
        messages: List[Dict[str, Any]],
        max_seq_length: Optional[int] = None,
    ) -> GenericOutputs:
        """
        Preprocesses messages for generation.

        Args:
            messages (List[Dict[str, Any]]): The list of messages to process.
            max_seq_length (Optional[int]): The maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: The processed input IDs tensor.
        """
        while messages and messages[-1]["role"] != "assistant":
            messages.pop()

        text = self.chat_template(messages[:-1])
        text_pair = self.chat_template(messages[-1:])
        outputs = self.generation(
            text=text,
            text_pair=text_pair,
            max_seq_length=max_seq_length,
        )
        return GenericOutputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            input_ids_label=outputs.input_ids_label,
            attention_mask_label=outputs.attention_mask_label,
        )


class HfTextClassificationProcessor:
    """
    Processor for text classification tasks.

    Args:
        tokenizer (PreTrainedTokenizer): The tokenizer to use for text classification.
        max_seq_length (int, optional): Maximum sequence length. Defaults to 128.
        source_type_id (int, optional): Source type ID. Defaults to 0.
        target_type_id (int, optional): Target type ID. Defaults to 1.
        position_start_id (int, optional): Start position ID. Defaults to 0.
    """

    def __init__(
        self,
        tokenizer: PreTrainedTokenizer,
        max_seq_length: Optional[int] = 128,
        source_type_id: Optional[int] = 0,
        target_type_id: Optional[int] = 1,
        position_start_id: Optional[int] = 0,
    ):
        self.tokenizer = tokenizer
        self.max_seq_length = max_seq_length
        self.pad_token = self.tokenizer.pad_token
        self.sep_token = self.tokenizer.sep_token
        self.cls_token = self.tokenizer.cls_token
        self.mask_token = self.tokenizer.mask_token
        self.pad_token_id = self.tokenizer.pad_token_id
        self.source_type_id = source_type_id
        self.target_type_id = target_type_id
        self.position_start_id = position_start_id

    def classification(
        self,
        text: str,
        text_pair: Optional[str] = None,
        max_seq_length: Optional[int] = None,
    ):
        """
        Generate inputs for text classification.

        Args:
            text (str): The input text.
            text_pair (str, optional): The paired text. Defaults to None.
            max_seq_length (int, optional): Maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: The generated input tokens, token type IDs, attention mask, and position IDs.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )

        tokens = self.tokenizer.tokenize(str(text))
        if text_pair is None:
            if self.cls_token is not None:
                tokens = tokens[: max_seq_length - 2]
                tokens = [self.cls_token] + tokens + [self.sep_token]
            else:
                tokens = tokens[: max_seq_length - 1]
                tokens = tokens + [self.sep_token]
            input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
            token_type_ids = [self.source_type_id] * len(input_ids)
            attention_mask = [1] * len(input_ids)
        else:
            tokens_pair = self.tokenizer.tokenize(str(text_pair))
            if self.cls_token is not None:
                truncate_sequence_pair(tokens, tokens_pair, max_seq_length - 3)
                token_type_ids = (
                    [self.source_type_id]
                    + [self.source_type_id] * len(tokens)
                    + [self.source_type_id]
                    + [self.target_type_id] * len(tokens_pair)
                    + [self.target_type_id]
                )
                tokens = (
                    [self.cls_token]
                    + tokens
                    + [self.sep_token]
                    + tokens_pair
                    + [self.sep_token]
                )
            else:
                truncate_sequence_pair(tokens, tokens_pair, max_seq_length - 2)
                token_type_ids = [self.source_type_id] * len(tokens) + [
                    self.target_type_id
                ] * len(tokens_pair)
                tokens = tokens + [self.sep_token] + tokens_pair + [self.sep_token]
            input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
            attention_mask = [1] * len(input_ids)

        padding = [0] * (max_seq_length - len(input_ids))
        input_ids += len(padding) * [self.pad_token_id]
        attention_mask += padding
        token_type_ids += len(padding) * [self.target_type_id]

        assert len(input_ids) == max_seq_length
        assert len(attention_mask) == max_seq_length
        assert len(token_type_ids) == max_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            token_type_ids=torch.tensor(token_type_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
            position_ids=torch.tensor(
                list(
                    range(
                        self.position_start_id,
                        self.position_start_id + max_seq_length,
                    )
                ),
                dtype=torch.long,
            ),
        )


class HfImageClassificationProcessor:
    """
    Processor for image classification tasks.
    """

    def __init__(
        self,
        vision_processor: BaseImageProcessor,
    ):
        """
        Initialize the HfImageClassificationProcessor.

        Args:
            vision_processor (BaseImageProcessor): The vision processor object used for image transformations.
        """
        self.vision_processor = vision_processor

    def classification(
        self,
        image: Union[Image.Image, str],
    ):
        """
        Perform image classification on the given image.

        Args:
            image (Image.Image): The input image.

        Returns:
            GenericOutputs: The output of the image classification, including pixel values.
        """
        if isinstance(image, str):
            image = Image.open(image)

        pixel_values = self.vision_processor.preprocess(
            image, return_tensors="pt"
        ).pixel_values[0]

        return GenericOutputs(
            pixel_values=torch.tensor(pixel_values),
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.cli import hf_endpoint_url

# pretrained infos
pretrained_qwen_infos = {
    "qwen3-4b-thinking": {
        "config": hf_endpoint_url(
            "/Qwen/Qwen3-4B-Thinking-2507/resolve/main/config.json"
        ),
        "tokenizer": hf_endpoint_url(
            "/Qwen/Qwen3-4B-Thinking-2507/resolve/main/tokenizer.json"
        ),
        "tokenizer_config": hf_endpoint_url(
            "/Qwen/Qwen3-4B-Thinking-2507/resolve/main/tokenizer_config.json"
        ),
        "chat_template": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-3B-Instruct/resolve/main/chat_template.json"
        ),
        "weight": [
            hf_endpoint_url(
                f"/Qwen/Qwen3-4B-Thinking-2507/resolve/main/model-{str(i).rjust(5, '0')}-of-00003.safetensors"
            )
            for i in range(1, 4)
        ],
    },
    "qwen3-14b": {
        "config": hf_endpoint_url("/Qwen/Qwen3-14B/resolve/main/config.json"),
        "tokenizer": hf_endpoint_url("/Qwen/Qwen3-14B/resolve/main/tokenizer.json"),
        "tokenizer_config": hf_endpoint_url(
            "/Qwen/Qwen3-14B/resolve/main/tokenizer_config.json"
        ),
        "weight": [
            hf_endpoint_url(
                f"/Qwen/Qwen3-14B/resolve/main/model-{str(i).rjust(5, '0')}-of-00008.safetensors"
            )
            for i in range(1, 9)
        ],
    },
    "qwen3-32b": {
        "config": hf_endpoint_url("/Qwen/Qwen3-32B/resolve/main/config.json"),
        "tokenizer": hf_endpoint_url("/Qwen/Qwen3-32B/resolve/main/tokenizer.json"),
        "tokenizer_config": hf_endpoint_url(
            "/Qwen/Qwen3-32B/resolve/main/tokenizer_config.json"
        ),
        "weight": [
            hf_endpoint_url(
                f"/Qwen/Qwen3-32B/resolve/main/model-{str(i).rjust(5, '0')}-of-00017.safetensors"
            )
            for i in range(1, 18)
        ],
    },
    "qwen2_5-vl-3b-instruct": {
        "config": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-3B-Instruct/resolve/main/config.json"
        ),
        "tokenizer": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-3B-Instruct/resolve/main/tokenizer.json"
        ),
        "vision_config": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-3B-Instruct/resolve/main/preprocessor_config.json"
        ),
        "tokenizer_config": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-3B-Instruct/resolve/main/tokenizer_config.json"
        ),
        "chat_template": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-3B-Instruct/resolve/main/chat_template.json"
        ),
        "weight": [
            hf_endpoint_url(
                f"/Qwen/Qwen2.5-VL-3B-Instruct/resolve/main/model-{str(i).rjust(5, '0')}-of-00002.safetensors"
            )
            for i in range(1, 3)
        ],
    },
    "qwen2_5-vl-7b-instruct": {
        "config": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-7B-Instruct/resolve/main/config.json"
        ),
        "tokenizer": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-7B-Instruct/resolve/main/tokenizer.json"
        ),
        "vision_config": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-7B-Instruct/resolve/main/preprocessor_config.json"
        ),
        "tokenizer_config": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-7B-Instruct/resolve/main/tokenizer_config.json"
        ),
        "chat_template": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-7B-Instruct/resolve/main/chat_template.json"
        ),
        "weight": [
            hf_endpoint_url(
                f"/Qwen/Qwen2.5-VL-7B-Instruct/resolve/main/model-{str(i).rjust(5, '0')}-of-00005.safetensors"
            )
            for i in range(1, 6)
        ],
    },
    "qwen2_5-vl-32b-instruct": {
        "config": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-32B-Instruct/resolve/main/config.json"
        ),
        "tokenizer": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-32B-Instruct/resolve/main/tokenizer.json"
        ),
        "vision_config": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-32B-Instruct/resolve/main/preprocessor_config.json"
        ),
        "tokenizer_config": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-32B-Instruct/resolve/main/tokenizer_config.json"
        ),
        "chat_template": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-32B-Instruct/resolve/main/chat_template.json"
        ),
        "weight": [
            hf_endpoint_url(
                f"/Qwen/Qwen2.5-VL-32B-Instruct/resolve/main/model-{str(i).rjust(5, '0')}-of-00018.safetensors"
            )
            for i in range(1, 19)
        ],
    },
    "qwen2_5-vl-72b-instruct": {
        "config": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-72B-Instruct/resolve/main/config.json"
        ),
        "tokenizer": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-72B-Instruct/resolve/main/tokenizer.json"
        ),
        "vision_config": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-72B-Instruct/resolve/main/preprocessor_config.json"
        ),
        "tokenizer_config": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-72B-Instruct/resolve/main/tokenizer_config.json"
        ),
        "chat_template": hf_endpoint_url(
            "/Qwen/Qwen2.5-VL-72B-Instruct/resolve/main/chat_template.json"
        ),
        "weight": [
            hf_endpoint_url(
                f"/Qwen/Qwen2.5-VL-72B-Instruct/resolve/main/model-{str(i).rjust(5, '0')}-of-00038.safetensors"
            )
            for i in range(1, 39)
        ],
    },
}

pretrained_qwen_extensions_infos = {}

import unitorch.cli.models.qwen.modeling
import unitorch.cli.models.qwen.modeling_vl
import unitorch.cli.models.qwen.processing
import unitorch.cli.models.qwen.processing_vl



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import random
from PIL import Image
from functools import partial
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union

import numpy as np
import torch

from transformers import (
    LlamaTokenizer,
    LlamaTokenizerFast,
    CLIPImageProcessor,
    SiglipImageProcessor,
    AddedToken,
)
from unitorch.utils import (
    pop_value,
    truncate_sequence_pair,
    read_json_file,
    get_added_token,
)
from unitorch.models import (
    HfImageClassificationProcessor,
    HfTextClassificationProcessor,
    HfTextGenerationProcessor,
    GenericOutputs,
)


class LlavaMistralClipProcessor(
    HfTextClassificationProcessor,
    HfTextGenerationProcessor,
    HfImageClassificationProcessor,
):
    def __init__(
        self,
        tokenizer_file: str,
        tokenizer_config: Optional[str] = None,
        special_tokens_map: Optional[str] = None,
        chat_template: Optional[str] = None,
        vision_config_path: Optional[str] = None,
        max_seq_length: Optional[int] = 128,
        max_gen_seq_length: Optional[int] = 48,
    ):
        """
        Initialize the LlamaProcessor.

        Args:
            vocab_file (str): Path to the vocabulary file.
            max_seq_length (int, optional): Maximum sequence length for text classification. Defaults to 128.
            max_gen_seq_length (int, optional): Maximum sequence length for text generation. Defaults to 48.
        """
        tokenizer_config = read_json_file(tokenizer_config) if tokenizer_config else {}
        special_tokens_map = (
            read_json_file(special_tokens_map) if special_tokens_map else {}
        )
        added_tokens_decoder = tokenizer_config.pop("added_tokens_decoder", {})
        tokenizer_config = {
            k: (
                get_added_token(v)
                if isinstance(v, dict) and v.get("__type") == "AddedToken"
                else v
            )
            for k, v in tokenizer_config.items()
        }

        tokenizer = LlamaTokenizerFast(
            tokenizer_file=tokenizer_file,
            **tokenizer_config,
        )
        for idx, spec in added_tokens_decoder.items():
            token = spec["content"]
            tokenizer.added_tokens_decoder[idx] = get_added_token(spec)
            tokenizer.added_tokens_encoder[token] = idx

        special_tokens = {}
        for name, spec in special_tokens_map.items():
            special_tokens[name] = get_added_token(spec)
        tokenizer.add_special_tokens(special_tokens)

        if chat_template:
            tokenizer.chat_template = read_json_file(chat_template)["chat_template"]
        tokenizer.cls_token = tokenizer.bos_token
        tokenizer.sep_token = tokenizer.eos_token
        tokenizer.pad_token = tokenizer.unk_token
        tokenizer.cls_token_id = tokenizer.bos_token_id
        tokenizer.sep_token_id = tokenizer.eos_token_id
        tokenizer.pad_token_id = tokenizer.unk_token_id
        HfTextClassificationProcessor.__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
        )
        HfTextGenerationProcessor.__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )

        if vision_config_path is not None:
            vision_processor = CLIPImageProcessor.from_json_file(vision_config_path)
        else:
            vision_processor = CLIPImageProcessor.from_pretrained(
                "openai/clip-vit-base-patch32"
            )
        HfImageClassificationProcessor.__init__(
            self,
            vision_processor=vision_processor,
        )

    def chat_template(
        self,
        messages: List[Dict[str, Any]],
    ):
        text = self.tokenizer.apply_chat_template(
            messages,
            tokenize=False,
            add_generation_prompt=True,
        )
        return text

    def image_classification(
        self,
        image: Union[Image.Image, str],
    ):
        """
        Performs image classification.

        Args:
            image (PIL.Image.Image): The input image.

        Returns:
            GenericOutputs: An object containing the processed inputs.
        """
        outputs = HfImageClassificationProcessor.classification(
            self,
            image=image,
        )

        return GenericOutputs(
            pixel_values=outputs.pixel_values,
        )

    def classification(
        self,
        text: str,
        image: Union[Image.Image, str],
        text_pair: Optional[str] = None,
        max_seq_length: Optional[int] = None,
    ):
        """
        Process text for classification.

        Args:
            text (str): Input text.
            text_pair (str, optional): Input text pair. Defaults to None.
            max_seq_length (int, optional): Maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids and attention_mask tensors.
        """
        assert "<image>" in text and not text.endswith("<image>")
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )

        tokens = self.tokenizer.tokenize(str(text))
        if text_pair is None:
            tokens = tokens[:max_seq_length]
            input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
        else:
            tokens_pair = self.tokenizer.tokenize(str(text_pair))
            truncate_sequence_pair(tokens, tokens_pair, max_seq_length)
            tokens = tokens + tokens_pair
            input_ids = self.tokenizer.convert_tokens_to_ids(tokens)

        padding = [0] * (max_seq_length - len(input_ids))
        attention_mask = [0] * len(padding) + [1] * len(input_ids)
        input_ids = len(padding) * [self.pad_token_id] + input_ids

        pixel_outputs = self.image_classification(
            image=image,
        )

        assert len(input_ids) == max_seq_length
        assert len(attention_mask) == max_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
            pixel_values=pixel_outputs.pixel_values,
        )

    def generation_inputs(
        self,
        text: str,
        image: Union[Image.Image, str],
        max_seq_length: Optional[int] = None,
    ):
        """
        Process text for generation inputs.

        Args:
            text (str): Input text.
            max_seq_length (int, optional): Maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids tensor.
        """
        assert "<image>" in text and not text.endswith("<image>")
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )
        tokens = [self.bos_token] + self.tokenizer.tokenize(str(text))[
            1 - max_seq_length :
        ]
        padding = [self.pad_token] * (max_seq_length - len(tokens))
        attention_mask = [0] * len(padding) + [1] * len(tokens)
        tokens = padding + tokens
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)

        pixel_outputs = self.image_classification(
            image=image,
        )

        assert len(input_ids) == max_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
            pixel_values=pixel_outputs.pixel_values,
        )

    def generation_labels(
        self,
        text: str,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Process text for generation labels.

        Args:
            text (str): Input text.
            max_gen_seq_length (int, optional): Maximum generation sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids and attention_mask tensors.
        """
        max_gen_seq_length = pop_value(
            max_gen_seq_length,
            self.max_gen_seq_length,
        )
        tokens = self.tokenizer.tokenize(str(text))[: max_gen_seq_length - 1] + [
            self.eos_token
        ]
        padding = [self.pad_token] * (max_gen_seq_length - len(tokens))
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
        attention_mask = [1] * len(input_ids)

        padding = [0] * (max_gen_seq_length - len(input_ids))
        input_ids += [self.pad_token_id] * len(padding)
        attention_mask += padding

        assert len(input_ids) == max_gen_seq_length
        assert len(attention_mask) == max_gen_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
        )

    def generation(
        self,
        text: str,
        image: Union[Image.Image, str],
        text_pair: str,
        max_seq_length: Optional[int] = None,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Process text for generation.

        Args:
            text (str): Input text.
            text_pair (str): Input text pair.
            max_seq_length (int, optional): Maximum sequence length. Defaults to None.
            max_gen_seq_length (int, optional): Maximum generation sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids, attention_mask, input_ids_label, and attention_mask_label tensors.
        """
        assert "<image>" in text and not text.endswith("<image>")
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )
        max_gen_seq_length = pop_value(
            max_gen_seq_length,
            self.max_gen_seq_length,
        )

        tokens = [self.bos_token] + self.tokenizer.tokenize(str(text))[
            1 - max_seq_length :
        ]
        tokens_pair = self.tokenizer.tokenize(str(text_pair))[
            : max_gen_seq_length - 1
        ] + [self.eos_token]
        padding_a = [self.pad_token] * (max_seq_length - len(tokens))
        padding_b = [self.pad_token] * (max_gen_seq_length - len(tokens_pair))
        attention_mask = (
            [0] * len(padding_a)
            + [1] * (len(tokens) + len(tokens_pair))
            + [0] * len(padding_b)
        )
        _tokens = padding_a + tokens + tokens_pair + padding_b
        input_ids = self.tokenizer.convert_tokens_to_ids(_tokens)

        tokens_label = tokens_pair + [self.pad_token] * (
            max_gen_seq_length - len(tokens_pair) + 1
        )
        input_ids_label = self.tokenizer.convert_tokens_to_ids(tokens_label)
        input_ids_label = [0] * (max_seq_length - 1) + input_ids_label
        attention_mask_label = [1] * len(tokens_pair) + [0] * (
            max_gen_seq_length - len(tokens_pair) + 1
        )
        attention_mask_label = [0] * (max_seq_length - 1) + attention_mask_label

        pixel_outputs = self.image_classification(
            image=image,
        )

        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
            pixel_values=pixel_outputs.pixel_values,
            input_ids_label=torch.tensor(input_ids_label, dtype=torch.long),
            attention_mask_label=torch.tensor(attention_mask_label, dtype=torch.long),
        )

    def messages_generation(
        self,
        messages: List[Dict[str, Any]],
        max_seq_length: Optional[int] = None,
    ) -> GenericOutputs:
        """
        Preprocesses messages for generation.

        Args:
            messages (List[Dict[str, Any]]): The list of messages to process.
            max_seq_length (Optional[int]): The maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: The processed input IDs tensor.
        """
        while messages and messages[-1]["role"] != "assistant":
            messages.pop()

        text = self.chat_template(messages[:-1])
        text_pair = self.chat_template(messages[-1:])
        outputs = self.generation(
            text=text,
            text_pair=text_pair,
            max_seq_length=max_seq_length,
        )
        return GenericOutputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            input_ids_label=outputs.input_ids_label,
            attention_mask_label=outputs.attention_mask_label,
        )


class LlavaLlamaSiglipProcessor(
    HfTextClassificationProcessor,
    HfTextGenerationProcessor,
):
    def __init__(
        self,
        tokenizer_file: str,
        tokenizer_config: Optional[str] = None,
        special_tokens_map: Optional[str] = None,
        chat_template: Optional[str] = None,
        vision_config_path: Optional[str] = None,
        max_seq_length: Optional[int] = 128,
        max_gen_seq_length: Optional[int] = 48,
    ):
        """
        Initialize the LlamaProcessor.

        Args:
            vocab_file (str): Path to the vocabulary file.
            max_seq_length (int, optional): Maximum sequence length for text classification. Defaults to 128.
            max_gen_seq_length (int, optional): Maximum sequence length for text generation. Defaults to 48.
        """
        tokenizer_config = read_json_file(tokenizer_config) if tokenizer_config else {}
        special_tokens_map = (
            read_json_file(special_tokens_map) if special_tokens_map else {}
        )
        added_tokens_decoder = tokenizer_config.pop("added_tokens_decoder", {})
        tokenizer_config = {
            k: (
                get_added_token(v)
                if isinstance(v, dict) and v.get("__type") == "AddedToken"
                else v
            )
            for k, v in tokenizer_config.items()
        }
        tokenizer = LlamaTokenizerFast(
            tokenizer_file=tokenizer_file,
            **tokenizer_config,
        )

        for idx, spec in added_tokens_decoder.items():
            token = spec["content"]
            tokenizer.added_tokens_decoder[idx] = get_added_token(spec)
            tokenizer.added_tokens_encoder[token] = idx

        special_tokens = {}
        for name, spec in special_tokens_map.items():
            special_tokens[name] = get_added_token(spec)
        tokenizer.add_special_tokens(special_tokens)

        if chat_template:
            tokenizer.chat_template = read_json_file(chat_template)["chat_template"]
        tokenizer.cls_token = tokenizer.bos_token
        tokenizer.sep_token = tokenizer.eos_token
        tokenizer.cls_token_id = tokenizer.bos_token_id
        tokenizer.sep_token_id = tokenizer.eos_token_id
        tokenizer.pad_token_id = tokenizer.convert_tokens_to_ids(tokenizer.pad_token)
        HfTextClassificationProcessor.__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
        )
        HfTextGenerationProcessor.__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )

        if vision_config_path is not None:
            self.vision_processor = SiglipImageProcessor.from_json_file(
                vision_config_path
            )
        else:
            self.vision_processor = None

    def chat_template(
        self,
        messages: List[Dict[str, Any]],
    ):
        text = self.tokenizer.apply_chat_template(
            messages,
            tokenize=False,
            add_generation_prompt=True,
        )
        return text

    def image_classification(
        self,
        image: Union[Image.Image, str],
    ):
        """
        Performs image classification.

        Args:
            image (PIL.Image.Image): The input image.

        Returns:
            GenericOutputs: An object containing the processed inputs.
        """
        pixel_values = self.vision_processor.preprocess(
            image, return_tensors="pt"
        ).pixel_values[0]

        return GenericOutputs(
            pixel_values=pixel_values,
        )

    def classification(
        self,
        text: str,
        image: Union[Image.Image, str],
        text_pair: Optional[str] = None,
        max_seq_length: Optional[int] = None,
    ):
        """
        Process text for classification.

        Args:
            text (str): Input text.
            text_pair (str, optional): Input text pair. Defaults to None.
            max_seq_length (int, optional): Maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids and attention_mask tensors.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )

        tokens = self.tokenizer.tokenize(str(text))
        if text_pair is None:
            tokens = tokens[:max_seq_length]
            input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
        else:
            tokens_pair = self.tokenizer.tokenize(str(text_pair))
            truncate_sequence_pair(tokens, tokens_pair, max_seq_length)
            tokens = tokens + tokens_pair
            input_ids = self.tokenizer.convert_tokens_to_ids(tokens)

        padding = [0] * (max_seq_length - len(input_ids))
        attention_mask = [0] * len(padding) + [1] * len(input_ids)
        input_ids = len(padding) * [self.pad_token_id] + input_ids

        pixel_values = self.vision_processor.preprocess(
            image, return_tensors="pt"
        ).pixel_values[0]

        assert len(input_ids) == max_seq_length
        assert len(attention_mask) == max_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
            pixel_values=pixel_values,
        )

    def generation_inputs(
        self,
        text: str,
        image: Union[Image.Image, str],
        max_seq_length: Optional[int] = None,
    ):
        """
        Process text for generation inputs.

        Args:
            text (str): Input text.
            max_seq_length (int, optional): Maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids tensor.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )
        tokens = [self.bos_token] + self.tokenizer.tokenize(str(text))[
            1 - max_seq_length :
        ]
        padding = [self.pad_token] * (max_seq_length - len(tokens))
        attention_mask = [0] * len(padding) + [1] * len(tokens)
        tokens = padding + tokens
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)

        pixel_values = self.vision_processor.preprocess(
            image, return_tensors="pt"
        ).pixel_values[0]

        assert len(input_ids) == max_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
            pixel_values=pixel_values,
        )

    def generation_labels(
        self,
        text: str,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Process text for generation labels.

        Args:
            text (str): Input text.
            max_gen_seq_length (int, optional): Maximum generation sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids and attention_mask tensors.
        """
        max_gen_seq_length = pop_value(
            max_gen_seq_length,
            self.max_gen_seq_length,
        )
        tokens = self.tokenizer.tokenize(str(text))[: max_gen_seq_length - 1] + [
            self.eos_token
        ]
        padding = [self.pad_token] * (max_gen_seq_length - len(tokens))
        input_ids = self.tokenizer.convert_tokens_to_ids(tokens)
        attention_mask = [1] * len(input_ids)

        padding = [0] * (max_gen_seq_length - len(input_ids))
        input_ids += [self.pad_token_id] * len(padding)
        attention_mask += padding

        assert len(input_ids) == max_gen_seq_length
        assert len(attention_mask) == max_gen_seq_length
        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
        )

    def generation(
        self,
        text: str,
        image: Union[Image.Image, str],
        text_pair: str,
        max_seq_length: Optional[int] = None,
        max_gen_seq_length: Optional[int] = None,
    ):
        """
        Process text for generation.

        Args:
            text (str): Input text.
            text_pair (str): Input text pair.
            max_seq_length (int, optional): Maximum sequence length. Defaults to None.
            max_gen_seq_length (int, optional): Maximum generation sequence length. Defaults to None.

        Returns:
            GenericOutputs: Processed input_ids, attention_mask, input_ids_label, and attention_mask_label tensors.
        """
        max_seq_length = pop_value(
            max_seq_length,
            self.max_seq_length,
        )
        max_gen_seq_length = pop_value(
            max_gen_seq_length,
            self.max_gen_seq_length,
        )

        tokens = [self.bos_token] + self.tokenizer.tokenize(str(text))[
            1 - max_seq_length :
        ]
        tokens_pair = self.tokenizer.tokenize(str(text_pair))[
            : max_gen_seq_length - 1
        ] + [self.eos_token]
        padding_a = [self.pad_token] * (max_seq_length - len(tokens))
        padding_b = [self.pad_token] * (max_gen_seq_length - len(tokens_pair))
        attention_mask = (
            [0] * len(padding_a)
            + [1] * (len(tokens) + len(tokens_pair))
            + [0] * len(padding_b)
        )
        _tokens = padding_a + tokens + tokens_pair + padding_b
        input_ids = self.tokenizer.convert_tokens_to_ids(_tokens)

        tokens_label = tokens_pair + [self.pad_token] * (
            max_gen_seq_length - len(tokens_pair) + 1
        )
        input_ids_label = self.tokenizer.convert_tokens_to_ids(tokens_label)
        input_ids_label = [0] * (max_seq_length - 1) + input_ids_label
        attention_mask_label = [1] * len(tokens_pair) + [0] * (
            max_gen_seq_length - len(tokens_pair) + 1
        )
        attention_mask_label = [0] * (max_seq_length - 1) + attention_mask_label

        pixel_values = self.vision_processor.preprocess(
            image, return_tensors="pt"
        ).pixel_values[0]

        return GenericOutputs(
            input_ids=torch.tensor(input_ids, dtype=torch.long),
            attention_mask=torch.tensor(attention_mask, dtype=torch.long),
            pixel_values=pixel_values,
            input_ids_label=torch.tensor(input_ids_label, dtype=torch.long),
            attention_mask_label=torch.tensor(attention_mask_label, dtype=torch.long),
        )

    def messages_generation(
        self,
        messages: List[Dict[str, Any]],
        max_seq_length: Optional[int] = None,
    ) -> GenericOutputs:
        """
        Preprocesses messages for generation.

        Args:
            messages (List[Dict[str, Any]]): The list of messages to process.
            max_seq_length (Optional[int]): The maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: The processed input IDs tensor.
        """
        while messages and messages[-1]["role"] != "assistant":
            messages.pop()

        text = self.chat_template(messages[:-1])
        text_pair = self.chat_template(messages[-1:])
        outputs = self.generation(
            text=text,
            text_pair=text_pair,
            max_seq_length=max_seq_length,
        )
        return GenericOutputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            input_ids_label=outputs.input_ids_label,
            attention_mask_label=outputs.attention_mask_label,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import torch
import diffusers
import numpy as np
from typing import Optional, Union, List, Dict, Any, Callable
from diffusers.pipelines import (
    StableDiffusion3Img2ImgPipeline,
    StableDiffusion3InpaintPipeline,
)
from diffusers.pipelines.stable_diffusion_3.pipeline_stable_diffusion_3_img2img import (
    SD3Transformer2DModel,
    FlowMatchEulerDiscreteScheduler,
    AutoencoderKL,
    CLIPTextModelWithProjection,
    CLIPTokenizer,
    T5EncoderModel,
    T5TokenizerFast,
    VaeImageProcessor,
)
from diffusers.pipelines.wan.pipeline_wan_i2v import (
    AutoTokenizer,
    CLIPImageProcessor,
    AutoencoderKLWan,
    UMT5EncoderModel,
    WanTransformer3DModel,
    CLIPVisionModel,
    WanImageToVideoPipeline,
)
from diffusers.pipelines.qwenimage.pipeline_qwenimage_edit import (
    QwenImageEditPipeline,
    QwenImagePipelineOutput,
    PipelineImageInput,
    calculate_dimensions,
    calculate_shift,
    retrieve_timesteps,
)
from unitorch.models import GenericOutputs
from unitorch.utils.decorators import replace


@replace(
    diffusers.pipelines.stable_diffusion_3.pipeline_stable_diffusion_3_img2img.StableDiffusion3Img2ImgPipeline
)
class StableDiffusion3Img2ImgPipelineV2(StableDiffusion3Img2ImgPipeline):
    def __init__(
        self,
        transformer: SD3Transformer2DModel,
        scheduler: FlowMatchEulerDiscreteScheduler,
        vae: AutoencoderKL,
        text_encoder: CLIPTextModelWithProjection,
        tokenizer: CLIPTokenizer,
        text_encoder_2: CLIPTextModelWithProjection,
        tokenizer_2: CLIPTokenizer,
        text_encoder_3: T5EncoderModel,
        tokenizer_3: T5TokenizerFast,
    ):
        super().__init__(
            transformer=transformer,
            scheduler=scheduler,
            vae=vae,
            text_encoder=text_encoder,
            tokenizer=(
                tokenizer
                if tokenizer is not None
                else GenericOutputs(model_max_length=77)
            ),
            text_encoder_2=text_encoder_2,
            tokenizer_2=(
                tokenizer_2
                if tokenizer_2 is not None
                else GenericOutputs(model_max_length=77)
            ),
            text_encoder_3=text_encoder_3,
            tokenizer_3=(
                tokenizer_3
                if tokenizer_3 is not None
                else GenericOutputs(model_max_length=77)
            ),
        )


@replace(
    diffusers.pipelines.stable_diffusion_3.pipeline_stable_diffusion_3_inpaint.StableDiffusion3InpaintPipeline
)
class StableDiffusion3InpaintPipelineV2(StableDiffusion3InpaintPipeline):
    def __init__(
        self,
        transformer: SD3Transformer2DModel,
        scheduler: FlowMatchEulerDiscreteScheduler,
        vae: AutoencoderKL,
        text_encoder: CLIPTextModelWithProjection,
        tokenizer: CLIPTokenizer,
        text_encoder_2: CLIPTextModelWithProjection,
        tokenizer_2: CLIPTokenizer,
        text_encoder_3: T5EncoderModel,
        tokenizer_3: T5TokenizerFast,
    ):
        super().__init__(
            transformer=transformer,
            scheduler=scheduler,
            vae=vae,
            text_encoder=text_encoder,
            tokenizer=(
                tokenizer
                if tokenizer is not None
                else GenericOutputs(model_max_length=77)
            ),
            text_encoder_2=text_encoder_2,
            tokenizer_2=(
                tokenizer_2
                if tokenizer_2 is not None
                else GenericOutputs(model_max_length=77)
            ),
            text_encoder_3=text_encoder_3,
            tokenizer_3=(
                tokenizer_3
                if tokenizer_3 is not None
                else GenericOutputs(model_max_length=77)
            ),
        )


@replace(diffusers.pipelines.wan.pipeline_wan_i2v.WanImageToVideoPipeline)
class WanImageToVideoPipelineV2(WanImageToVideoPipeline):

    def check_inputs(
        self,
        prompt,
        negative_prompt,
        image,
        height,
        width,
        prompt_embeds=None,
        negative_prompt_embeds=None,
        image_embeds=None,
        callback_on_step_end_tensor_inputs=None,
        guidance_scale_2=None,
    ):
        pass


@replace(diffusers.pipelines.qwenimage.pipeline_qwenimage_edit.QwenImageEditPipeline)
class QwenImageEditPipelineV2(QwenImageEditPipeline):
    @torch.no_grad()
    def __call__(
        self,
        image: Optional[PipelineImageInput] = None,
        prompt: Union[str, List[str]] = None,
        negative_prompt: Union[str, List[str]] = None,
        true_cfg_scale: float = 4.0,
        height: Optional[int] = None,
        width: Optional[int] = None,
        num_inference_steps: int = 50,
        sigmas: Optional[List[float]] = None,
        guidance_scale: float = 1.0,
        num_images_per_prompt: int = 1,
        generator: Optional[Union[torch.Generator, List[torch.Generator]]] = None,
        latents: Optional[torch.Tensor] = None,
        prompt_embeds: Optional[torch.Tensor] = None,
        prompt_embeds_mask: Optional[torch.Tensor] = None,
        negative_prompt_embeds: Optional[torch.Tensor] = None,
        negative_prompt_embeds_mask: Optional[torch.Tensor] = None,
        output_type: Optional[str] = "pil",
        return_dict: bool = True,
        attention_kwargs: Optional[Dict[str, Any]] = None,
        callback_on_step_end: Optional[Callable[[int, int, Dict], None]] = None,
        callback_on_step_end_tensor_inputs: List[str] = ["latents"],
        max_sequence_length: int = 512,
    ):
        if not isinstance(image, torch.Tensor):
            image_size = image[0].size if isinstance(image, list) else image.size
        else:
            image_size = (width, height)
        calculated_width, calculated_height, _ = calculate_dimensions(
            1024 * 1024, image_size[0] / image_size[1]
        )
        height = height or calculated_height
        width = width or calculated_width

        multiple_of = self.vae_scale_factor * 2
        width = width // multiple_of * multiple_of
        height = height // multiple_of * multiple_of

        # 1. Check inputs. Raise error if not correct
        self.check_inputs(
            prompt,
            height,
            width,
            negative_prompt=negative_prompt,
            prompt_embeds=prompt_embeds,
            negative_prompt_embeds=negative_prompt_embeds,
            prompt_embeds_mask=prompt_embeds_mask,
            negative_prompt_embeds_mask=negative_prompt_embeds_mask,
            callback_on_step_end_tensor_inputs=callback_on_step_end_tensor_inputs,
            max_sequence_length=max_sequence_length,
        )

        self._guidance_scale = guidance_scale
        self._attention_kwargs = attention_kwargs
        self._current_timestep = None
        self._interrupt = False

        # 2. Define call parameters
        if prompt is not None and isinstance(prompt, str):
            batch_size = 1
        elif prompt is not None and isinstance(prompt, list):
            batch_size = len(prompt)
        else:
            batch_size = prompt_embeds.shape[0]

        device = self._execution_device
        # 3. Preprocess image
        if image is not None and not (
            isinstance(image, torch.Tensor) and image.size(1) == self.latent_channels
        ):
            image = self.image_processor.resize(
                image, calculated_height, calculated_width
            )
            prompt_image = image
            image = self.image_processor.preprocess(
                image, calculated_height, calculated_width
            )
            image = image.unsqueeze(2)

        has_neg_prompt = negative_prompt is not None or (
            negative_prompt_embeds is not None
            and negative_prompt_embeds_mask is not None
        )
        do_true_cfg = true_cfg_scale > 1 and has_neg_prompt
        prompt_embeds, prompt_embeds_mask = self.encode_prompt(
            image=prompt_image,
            prompt=prompt,
            prompt_embeds=prompt_embeds,
            prompt_embeds_mask=prompt_embeds_mask,
            device=device,
            num_images_per_prompt=num_images_per_prompt,
            max_sequence_length=max_sequence_length,
        )
        if do_true_cfg:
            negative_prompt_embeds, negative_prompt_embeds_mask = self.encode_prompt(
                image=prompt_image,
                prompt=negative_prompt,
                prompt_embeds=negative_prompt_embeds,
                prompt_embeds_mask=negative_prompt_embeds_mask,
                device=device,
                num_images_per_prompt=num_images_per_prompt,
                max_sequence_length=max_sequence_length,
            )

        # 4. Prepare latent variables
        num_channels_latents = self.transformer.config.in_channels // 4
        latents, image_latents = self.prepare_latents(
            image,
            batch_size * num_images_per_prompt,
            num_channels_latents,
            height,
            width,
            prompt_embeds.dtype,
            device,
            generator,
            latents,
        )
        img_shapes = [
            [
                (
                    1,
                    height // self.vae_scale_factor // 2,
                    width // self.vae_scale_factor // 2,
                ),
                (
                    1,
                    calculated_height // self.vae_scale_factor // 2,
                    calculated_width // self.vae_scale_factor // 2,
                ),
            ]
        ] * batch_size

        # 5. Prepare timesteps
        sigmas = (
            np.linspace(1.0, 1 / num_inference_steps, num_inference_steps)
            if sigmas is None
            else sigmas
        )
        image_seq_len = latents.shape[1]
        mu = calculate_shift(
            image_seq_len,
            self.scheduler.config.get("base_image_seq_len", 256),
            self.scheduler.config.get("max_image_seq_len", 4096),
            self.scheduler.config.get("base_shift", 0.5),
            self.scheduler.config.get("max_shift", 1.15),
        )
        timesteps, num_inference_steps = retrieve_timesteps(
            self.scheduler,
            num_inference_steps,
            device,
            sigmas=sigmas,
            mu=mu,
        )
        num_warmup_steps = max(
            len(timesteps) - num_inference_steps * self.scheduler.order, 0
        )
        self._num_timesteps = len(timesteps)

        # handle guidance
        if self.transformer.config.guidance_embeds:
            guidance = torch.full(
                [1], guidance_scale, device=device, dtype=torch.float32
            )
            guidance = guidance.expand(latents.shape[0])
        else:
            guidance = None

        if self.attention_kwargs is None:
            self._attention_kwargs = {}

        txt_seq_lens = (
            prompt_embeds_mask.sum(dim=1).tolist()
            if prompt_embeds_mask is not None
            else None
        )
        negative_txt_seq_lens = (
            negative_prompt_embeds_mask.sum(dim=1).tolist()
            if negative_prompt_embeds_mask is not None
            else None
        )

        # 6. Denoising loop
        self.scheduler.set_begin_index(0)
        with self.progress_bar(total=num_inference_steps) as progress_bar:
            for i, t in enumerate(timesteps):
                if self.interrupt:
                    continue

                self._current_timestep = t

                latent_model_input = latents
                if image_latents is not None:
                    latent_model_input = torch.cat([latents, image_latents], dim=1)

                # broadcast to batch dimension in a way that's compatible with ONNX/Core ML
                timestep = t.expand(latents.shape[0]).to(latents.dtype)
                with self.transformer.cache_context("cond"):
                    noise_pred = self.transformer(
                        hidden_states=latent_model_input,
                        timestep=timestep / 1000,
                        guidance=guidance,
                        encoder_hidden_states_mask=prompt_embeds_mask,
                        encoder_hidden_states=prompt_embeds,
                        img_shapes=img_shapes,
                        txt_seq_lens=txt_seq_lens,
                        attention_kwargs=self.attention_kwargs,
                        return_dict=False,
                    )[0]
                    noise_pred = noise_pred[:, : latents.size(1)]

                if do_true_cfg:
                    with self.transformer.cache_context("uncond"):
                        neg_noise_pred = self.transformer(
                            hidden_states=latent_model_input,
                            timestep=timestep / 1000,
                            guidance=guidance,
                            encoder_hidden_states_mask=negative_prompt_embeds_mask,
                            encoder_hidden_states=negative_prompt_embeds,
                            img_shapes=img_shapes,
                            txt_seq_lens=negative_txt_seq_lens,
                            attention_kwargs=self.attention_kwargs,
                            return_dict=False,
                        )[0]
                    neg_noise_pred = neg_noise_pred[:, : latents.size(1)]
                    comb_pred = neg_noise_pred + true_cfg_scale * (
                        noise_pred - neg_noise_pred
                    )

                    cond_norm = torch.norm(noise_pred, dim=-1, keepdim=True)
                    noise_norm = torch.norm(comb_pred, dim=-1, keepdim=True)
                    noise_pred = comb_pred * (cond_norm / noise_norm)

                # compute the previous noisy sample x_t -> x_t-1
                latents_dtype = latents.dtype
                latents = self.scheduler.step(
                    noise_pred, t, latents, return_dict=False
                )[0]

                if latents.dtype != latents_dtype:
                    if torch.backends.mps.is_available():
                        # some platforms (eg. apple mps) misbehave due to a pytorch bug: https://github.com/pytorch/pytorch/pull/99272
                        latents = latents.to(latents_dtype)

                if callback_on_step_end is not None:
                    callback_kwargs = {}
                    for k in callback_on_step_end_tensor_inputs:
                        callback_kwargs[k] = locals()[k]
                    callback_outputs = callback_on_step_end(self, i, t, callback_kwargs)

                    latents = callback_outputs.pop("latents", latents)
                    prompt_embeds = callback_outputs.pop("prompt_embeds", prompt_embeds)

                # call the callback, if provided
                if i == len(timesteps) - 1 or (
                    (i + 1) > num_warmup_steps and (i + 1) % self.scheduler.order == 0
                ):
                    progress_bar.update()

        self._current_timestep = None
        if output_type == "latent":
            image = latents
        else:
            latents = self._unpack_latents(
                latents, height, width, self.vae_scale_factor
            )
            latents = latents.to(self.vae.dtype)
            latents_mean = (
                torch.tensor(self.vae.config.latents_mean)
                .view(1, self.vae.config.z_dim, 1, 1, 1)
                .to(latents.device, latents.dtype)
            )
            latents_std = 1.0 / torch.tensor(self.vae.config.latents_std).view(
                1, self.vae.config.z_dim, 1, 1, 1
            ).to(latents.device, latents.dtype)
            latents = latents / latents_std + latents_mean
            image = self.vae.decode(latents, return_dict=False)[0][:, :, 0]
            image = self.image_processor.postprocess(image, output_type=output_type)

        # Offload all models
        self.maybe_free_model_hooks()

        if not return_dict:
            return (image,)

        return QwenImagePipelineOutput(images=image)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import random
from functools import partial
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union

import numpy as np
import torch
from transformers import Qwen2Tokenizer, AddedToken, Qwen2TokenizerFast
from unitorch.utils import (
    pop_value,
    truncate_sequence_pair,
    read_json_file,
    get_added_token,
)
from unitorch.models import (
    HfTextClassificationProcessor,
    HfLlmProcessor,
    HfTextGenerationProcessor,
    GenericOutputs,
)


class QWenProcessor(HfLlmProcessor):
    def __init__(
        self,
        tokenizer_file: str,
        tokenizer_config: Optional[str] = None,
        special_tokens_map: Optional[str] = None,
        chat_template: Optional[str] = None,
        max_seq_length: Optional[int] = 12800,
        max_gen_seq_length: Optional[int] = 512,
    ):
        """
        Initializes the ClipProcessor.

        Args:
            vocab_path (str): The path to the vocabulary file.
            merge_path (str): The path to the merge file.
            max_seq_length (int, optional): The maximum sequence length for text inputs. Defaults to 262144.
        """
        tokenizer_config = read_json_file(tokenizer_config) if tokenizer_config else {}
        special_tokens_map = (
            read_json_file(special_tokens_map) if special_tokens_map else {}
        )
        added_tokens_decoder = tokenizer_config.pop("added_tokens_decoder", {})
        tokenizer_config = {
            k: (
                get_added_token(v)
                if isinstance(v, dict) and v.get("__type") == "AddedToken"
                else v
            )
            for k, v in tokenizer_config.items()
        }
        tokenizer = Qwen2TokenizerFast(
            tokenizer_file=tokenizer_file,
            **tokenizer_config,
        )
        for idx, spec in added_tokens_decoder.items():
            token = spec["content"]
            tokenizer.added_tokens_decoder[idx] = get_added_token(spec)
            tokenizer.added_tokens_encoder[token] = idx

        special_tokens = {}
        for name, spec in special_tokens_map.items():
            special_tokens[name] = get_added_token(spec)
        tokenizer.add_special_tokens(special_tokens)
        if chat_template:
            tokenizer.chat_template = read_json_file(chat_template)["chat_template"]
        tokenizer.cls_token = tokenizer.bos_token
        tokenizer.sep_token = tokenizer.eos_token
        super().__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.models.peft.diffusers.modeling_stable import (
    StableLoraForText2ImageGeneration,
    StableLoraForImageInpainting,
)
from unitorch.models.peft.diffusers.modeling_stable_xl import (
    StableXLLoraForText2ImageGeneration,
    StableXLLoraForImageInpainting,
)
from unitorch.models.peft.diffusers.modeling_stable_3 import (
    Stable3LoraForText2ImageGeneration,
    Stable3LoraForImageInpainting,
)
from unitorch.models.peft.diffusers.modeling_stable_flux import (
    StableFluxLoraForText2ImageGeneration,
    StableFluxLoraForImageInpainting,
    StableFluxLoraForKontext2ImageGeneration,
    StableFluxDPOLoraForText2ImageGeneration,
    StableFluxDPOLoraForImageInpainting,
    StableFluxDPOLoraForKontext2ImageGeneration,
)
from unitorch.models.peft.diffusers.modeling_qwen_image import (
    QWenImageLoraForText2ImageGeneration,
    QWenImageLoraForImageEditing,
)
from unitorch.models.peft.diffusers.modeling_wan import (
    WanLoraForText2VideoGeneration,
    WanLoraForImage2VideoGeneration,
)



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

from unitorch.models.qwen.modeling import (
    QWen3ForGeneration,
)
from unitorch.models.qwen.modeling_vl import QWen2_5VLForGeneration
from unitorch.models.qwen.processing import QWenProcessor
from unitorch.models.qwen.processing_vl import QWenVLProcessor



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.

import os
import random
from functools import partial
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
from PIL import Image
import numpy as np
import torch
from transformers import (
    Qwen2VLImageProcessor,
    Qwen2Tokenizer,
    Qwen2TokenizerFast,
    Qwen2_5_VLProcessor,
)
from unitorch.utils import (
    pop_value,
    truncate_sequence_pair,
    read_json_file,
    get_added_token,
)
from unitorch.models import (
    HfTextClassificationProcessor,
    HfTextGenerationProcessor,
    HfLlmProcessor,
    HfImageClassificationProcessor,
    GenericOutputs,
)


class QWenVLProcessor(HfLlmProcessor):
    def __init__(
        self,
        tokenizer_file: str,
        vision_config_path: str,
        tokenizer_config: Optional[str] = None,
        special_tokens_map: Optional[str] = None,
        chat_template: Optional[str] = None,
        max_seq_length: Optional[int] = 1280,
        max_gen_seq_length: Optional[int] = 512,
    ):
        """
        Initializes the ClipProcessor.

        Args:
            vocab_path (str): The path to the vocabulary file.
            merge_path (str): The path to the merge file.
            max_seq_length (int, optional): The maximum sequence length for text inputs. Defaults to 262144.
        """
        tokenizer_config = read_json_file(tokenizer_config) if tokenizer_config else {}
        special_tokens_map = (
            read_json_file(special_tokens_map) if special_tokens_map else {}
        )
        added_tokens_decoder = tokenizer_config.pop("added_tokens_decoder", {})
        tokenizer_config = {
            k: (
                get_added_token(v)
                if isinstance(v, dict) and v.get("__type") == "AddedToken"
                else v
            )
            for k, v in tokenizer_config.items()
        }
        tokenizer = Qwen2TokenizerFast(
            tokenizer_file=tokenizer_file,
            **tokenizer_config,
        )
        for idx, spec in added_tokens_decoder.items():
            token = spec["content"]
            tokenizer.added_tokens_decoder[idx] = get_added_token(spec)
            tokenizer.added_tokens_encoder[token] = idx

        special_tokens = {}
        for name, spec in special_tokens_map.items():
            if not isinstance(spec, dict or str):
                continue
            special_tokens[name] = get_added_token(spec)
        tokenizer.add_special_tokens(special_tokens)
        if chat_template:
            tokenizer.chat_template = read_json_file(chat_template)["chat_template"]
        tokenizer.cls_token = tokenizer.bos_token
        tokenizer.sep_token = tokenizer.eos_token

        self.image_token = (
            "<|image_pad|>"
            if not hasattr(tokenizer, "image_token")
            else tokenizer.image_token
        )
        self.video_token = (
            "<|video_pad|>"
            if not hasattr(tokenizer, "video_token")
            else tokenizer.video_token
        )
        self.image_token_id = (
            tokenizer.image_token_id
            if getattr(tokenizer, "image_token_id", None)
            else tokenizer.convert_tokens_to_ids(self.image_token)
        )
        self.video_token_id = (
            tokenizer.video_token_id
            if getattr(tokenizer, "video_token_id", None)
            else tokenizer.convert_tokens_to_ids(self.video_token)
        )
        self.vision_processor = Qwen2VLImageProcessor.from_json_file(vision_config_path)

        super().__init__(
            self,
            tokenizer=tokenizer,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )

    def processing_images(
        self,
        images: Union[Image.Image, str, List[Image.Image], List[str]],
    ):
        """
        Process images for classification.

        Args:
            images (Image.Image, str, List[Image.Image], List[str]): Input image or list of images.

        Returns:
            GenericOutputs: Processed outputs.
        """
        if isinstance(images, (Image.Image, str)):
            images = [images]
        images = [
            image if isinstance(image, Image.Image) else Image.open(image)
            for image in images
        ]
        outputs = self.vision_processor(images=images, return_tensors="pt")
        return outputs

    def classification(
        self,
        text: str,
        images: Union[Image.Image, str, List[Image.Image], List[str]],
        max_seq_length: Optional[int] = None,
    ):
        image_inputs = self.processing_images(images)
        image_index, image_merge_size = 0, self.vision_processor.merge_size**2
        image_grid_thw = image_inputs["image_grid_thw"] if image_inputs else None
        while self.image_token in text:
            num_image_tokens = image_grid_thw[image_index].prod() // image_merge_size
            text = text.replace(
                self.image_token,
                "<|placeholder|>" * num_image_tokens,
                1,
            )
            image_index += 1
        text = str(text).replace("<|placeholder|>", self.image_token)
        text_inputs = super().classification(text, max_seq_length=max_seq_length)

        return GenericOutputs(
            input_ids=torch.tensor(text_inputs.input_ids, dtype=torch.long),
            attention_mask=torch.tensor(text_inputs.attention_mask, dtype=torch.long),
            image_grid_thw=torch.tensor(image_grid_thw, dtype=torch.long),
            pixel_values=torch.tensor(image_inputs["pixel_values"]),
        )

    def generation_inputs(
        self,
        text: str,
        images: Union[Image.Image, str, List[Image.Image], List[str]],
        max_seq_length: Optional[int] = None,
    ):
        image_inputs = self.processing_images(images) if images else None
        image_index, image_merge_size = 0, self.vision_processor.merge_size**2
        image_grid_thw = image_inputs["image_grid_thw"] if image_inputs else None
        while self.image_token in text:
            num_image_tokens = image_grid_thw[image_index].prod() // image_merge_size
            text = text.replace(
                self.image_token,
                "<|placeholder|>" * num_image_tokens,
                1,
            )
            image_index += 1
        text = str(text).replace("<|placeholder|>", self.image_token)
        text_inputs = super().classification(text, max_seq_length=max_seq_length)
        return GenericOutputs(
            input_ids=torch.tensor(text_inputs.input_ids, dtype=torch.long),
            attention_mask=torch.tensor(text_inputs.attention_mask, dtype=torch.long),
            image_grid_thw=torch.tensor(image_grid_thw, dtype=torch.long),
            pixel_values=torch.tensor(image_inputs["pixel_values"]),
        )

    def generation(
        self,
        text: str,
        images: Union[Image.Image, str, List[Image.Image], List[str]],
        text_pair: str,
        max_seq_length: Optional[int] = None,
        max_gen_seq_length: Optional[int] = None,
    ):
        text, text_pair = str(text), str(text_pair)
        image_inputs = self.processing_images(images) if images else None
        image_index, image_merge_size = 0, self.vision_processor.merge_size**2
        image_grid_thw = image_inputs["image_grid_thw"] if image_inputs else None
        while self.image_token in text:
            num_image_tokens = image_grid_thw[image_index].prod() // image_merge_size
            text = text.replace(
                self.image_token,
                "<|placeholder|>" * num_image_tokens,
                1,
            )
            image_index += 1
        text = text.replace("<|placeholder|>", self.image_token)

        text_inputs = super().generation(
            text,
            text_pair=text_pair,
            max_seq_length=max_seq_length,
            max_gen_seq_length=max_gen_seq_length,
        )

        return GenericOutputs(
            input_ids=torch.tensor(text_inputs.input_ids, dtype=torch.long),
            attention_mask=torch.tensor(text_inputs.attention_mask, dtype=torch.long),
            image_grid_thw=(
                torch.tensor(image_grid_thw, dtype=torch.long)
                if image_grid_thw is not None
                else None
            ),
            pixel_values=(
                torch.tensor(image_inputs["pixel_values"])
                if image_inputs is not None
                else None
            ),
            input_ids_label=torch.tensor(text_inputs.input_ids_label, dtype=torch.long),
            attention_mask_label=torch.tensor(
                text_inputs.attention_mask_label, dtype=torch.long
            ),
        )

    def messages_generation(
        self,
        messages: List[Dict[str, Any]],
        images: Union[Image.Image, str, List[Image.Image], List[str]],
        max_seq_length: Optional[int] = None,
    ) -> GenericOutputs:
        """
        Preprocesses messages for generation.

        Args:
            messages (List[Dict[str, Any]]): The list of messages to process.
            max_seq_length (Optional[int]): The maximum sequence length. Defaults to None.

        Returns:
            GenericOutputs: The processed input IDs tensor.
        """
        while messages and messages[-1]["role"] != "assistant":
            messages.pop()

        text = self.chat_template(messages[:-1])
        text_pair = self.chat_template(messages[-1:])
        outputs = self.generation(
            text=text,
            images=images,
            text_pair=text_pair,
            max_seq_length=max_seq_length,
        )
        return GenericOutputs(
            input_ids=outputs.input_ids,
            attention_mask=outputs.attention_mask,
            image_grid_thw=outputs.image_grid_thw,
            pixel_values=outputs.pixel_values,
            input_ids_label=outputs.input_ids_label,
            attention_mask_label=outputs.attention_mask_label,
        )



# Copyright (c) FULIUCANSHENG.
# Licensed under the MIT License.


import unitorch.cli.fastapis.stable_3.text2image
import unitorch.cli.fastapis.stable_3.image2image
import unitorch.cli.fastapis.stable_3.inpainting
